@@ -10,7 +10,7 @@
  *   nor RFC2385 (which requires a kernel patch on 2.4 kernels).
  */
 
-char const *cvs_id_bgp = "$Id: bgp.c,v 1.7 2004/11/15 06:49:56 bodea Exp $";
+char const *cvs_id_bgp = "$Id: bgp.c,v 1.9 2004/12/16 08:49:52 bodea Exp $";
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -34,7 +34,7 @@ static struct bgp_route_list *bgp_insert_route(struct bgp_route_list *head,
     struct bgp_route_list *new);
 
 static void bgp_free_routes(struct bgp_route_list *routes);
-static char const *bgp_msg_type_str(u8 type);
+static char const *bgp_msg_type_str(uint8_t type);
 static int bgp_connect(struct bgp_peer *peer);
 static int bgp_handle_connect(struct bgp_peer *peer);
 static int bgp_write(struct bgp_peer *peer);
@@ -43,9 +43,10 @@ static int bgp_handle_input(struct bgp_peer *peer);
 static int bgp_send_open(struct bgp_peer *peer);
 static int bgp_send_keepalive(struct bgp_peer *peer);
 static int bgp_send_update(struct bgp_peer *peer);
-static int bgp_send_notification(struct bgp_peer *peer, u8 code, u8 subcode);
+static int bgp_send_notification(struct bgp_peer *peer, uint8_t code,
+    uint8_t subcode);
 
-static u16 our_as;
+static uint16_t our_as;
 static struct bgp_route_list *bgp_routes = 0;
 
 int bgp_configured = 0;
@@ -69,7 +70,7 @@ int bgp_setup(int as)
 	if (!((peer->outbuf = malloc(sizeof(*peer->outbuf)))
 	    && (peer->inbuf = malloc(sizeof(*peer->inbuf)))))
 	{
-	    LOG(0, 0, 0, 0, "Can't allocate buffers for bgp peer (%s)\n",
+	    LOG(0, 0, 0, "Can't allocate buffers for bgp peer (%s)\n",
 		strerror(errno));
 
 	    return 0;
@@ -89,7 +90,8 @@ int bgp_setup(int as)
 }
 
 /* start connection with a peer */
-int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold, int enable)
+int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive,
+    int hold, int enable)
 {
     struct hostent *h;
     int ibgp;
@@ -98,8 +100,8 @@ int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold
     char path_attrs[64];
     char *p = path_attrs;
     in_addr_t ip;
-    u32 metric = htonl(BGP_METRIC);
-    u32 no_export = htonl(BGP_COMMUNITY_NO_EXPORT);
+    uint32_t metric = htonl(BGP_METRIC);
+    uint32_t no_export = htonl(BGP_COMMUNITY_NO_EXPORT);
 
     if (!our_as)
 	return 0;
@@ -111,7 +113,7 @@ int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold
 
     if (!(h = gethostbyname(name)) || h->h_addrtype != AF_INET)
     {
-	LOG(0, 0, 0, 0, "Can't get address for BGP peer %s (%s)\n",
+	LOG(0, 0, 0, "Can't get address for BGP peer %s (%s)\n",
 	    name, h ? "no address" : hstrerror(h_errno));
 
 	return 0;
@@ -170,9 +172,9 @@ int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold
     {
 	/* just our AS */
 	struct {
-	    u8 type;
-	    u8 len;
-	    u16 value;
+	    uint8_t type;
+	    uint8_t len;
+	    uint16_t value;
 	} as_path = {
 	    BGP_PATH_ATTR_CODE_AS_PATH_AS_SEQUENCE,
 	    1,
@@ -204,7 +206,7 @@ int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold
 
     if (ibgp)
     {
-	u32 local_pref = htonl(BGP_LOCAL_PREF);
+	uint32_t local_pref = htonl(BGP_LOCAL_PREF);
 
 	/* LOCAL_PREF */
 	a.flags = BGP_PATH_ATTR_FLAG_TRANS;
@@ -225,7 +227,7 @@ int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold
 
     if (!(peer->path_attrs = malloc(peer->path_attr_len)))
     {
-	LOG(0, 0, 0, 0, "Can't allocate path_attrs for %s (%s)\n",
+	LOG(0, 0, 0, "Can't allocate path_attrs for %s (%s)\n",
 	    name, strerror(errno));
 
 	return 0;
@@ -233,7 +235,7 @@ int bgp_start(struct bgp_peer *peer, char *name, int as, int keepalive, int hold
 
     memcpy(peer->path_attrs, path_attrs, peer->path_attr_len);
 
-    LOG(4, 0, 0, 0, "Initiating BGP connection to %s (routing %s)\n",
+    LOG(4, 0, 0, "Initiating BGP connection to %s (routing %s)\n",
 	name, enable ? "enabled" : "suspended");
 
     /* we have at least one peer configured */
@@ -274,7 +276,7 @@ static void bgp_clear(struct bgp_peer *peer)
 	peer->state = peer->next_state;
 	peer->state_time = time_now;
 
-	LOG(4, 0, 0, 0, "BGP peer %s: state %s\n", peer->name,
+	LOG(4, 0, 0, "BGP peer %s: state %s\n", peer->name,
 	    bgp_state_str(peer->next_state));
     }
 }
@@ -282,14 +284,14 @@ static void bgp_clear(struct bgp_peer *peer)
 /* initiate a clean shutdown */
 void bgp_stop(struct bgp_peer *peer)
 {
-    LOG(4, 0, 0, 0, "Terminating BGP connection to %s\n", peer->name);
+    LOG(4, 0, 0, "Terminating BGP connection to %s\n", peer->name);
     bgp_send_notification(peer, BGP_ERR_CEASE, 0);
 }
 
 /* drop connection (if any) and set state to Disabled */
 void bgp_halt(struct bgp_peer *peer)
 {
-    LOG(4, 0, 0, 0, "Aborting BGP connection to %s\n", peer->name);
+    LOG(4, 0, 0, "Aborting BGP connection to %s\n", peer->name);
     peer->next_state = Disabled;
     bgp_clear(peer);
 }
@@ -324,7 +326,7 @@ static void bgp_set_retry(struct bgp_peer *peer)
 static void bgp_cidr(in_addr_t ip, in_addr_t mask, struct bgp_ip_prefix *pfx)
 {
     int i;
-    u32 b;
+    uint32_t b;
 
     /* convert to prefix notation */
     pfx->len = 32;
@@ -399,8 +401,8 @@ int bgp_add_route(in_addr_t ip, in_addr_t mask)
     /* insert into route list; sorted */
     if (!(r = malloc(sizeof(*r))))
     {
-	LOG(0, 0, 0, 0, "Can't allocate route for %s/%d (%s)\n",
-	    inet_toa(add.dest.prefix), add.dest.len, strerror(errno));
+	LOG(0, 0, 0, "Can't allocate route for %s/%d (%s)\n",
+	    fmtaddr(add.dest.prefix, 0), add.dest.len, strerror(errno));
 
 	return 0;
     }
@@ -413,8 +415,8 @@ int bgp_add_route(in_addr_t ip, in_addr_t mask)
 	if (bgp_peers[i].state == Established)
 	    bgp_peers[i].update_routes = 1;
 
-    LOG(4, 0, 0, 0, "Registered BGP route %s/%d\n", inet_toa(add.dest.prefix),
-	add.dest.len);
+    LOG(4, 0, 0, "Registered BGP route %s/%d\n",
+	fmtaddr(add.dest.prefix, 0), add.dest.len);
 
     return 1;
 }
@@ -462,8 +464,8 @@ int bgp_del_route(in_addr_t ip, in_addr_t mask)
 	if (bgp_peers[i].state == Established)
 	    bgp_peers[i].update_routes = 1;
 
-    LOG(4, 0, 0, 0, "Removed BGP route %s/%d\n", inet_toa(del.dest.prefix),
-	del.dest.len);
+    LOG(4, 0, 0, "Removed BGP route %s/%d\n",
+	fmtaddr(del.dest.prefix, 0), del.dest.len);
 
     return 1;
 }
@@ -482,7 +484,7 @@ void bgp_enable_routing(int enable)
 	    bgp_peers[i].update_routes = 1;
     }
 
-    LOG(4, 0, 0, 0, "%s BGP routing\n", enable ? "Enabled" : "Suspended");
+    LOG(4, 0, 0, "%s BGP routing\n", enable ? "Enabled" : "Suspended");
 }
 
 /* return a bitmask indicating if the socket should be added to the
@@ -588,7 +590,7 @@ int bgp_process(struct bgp_peer *peer, int readable, int writable)
     {
 	if (time_now > peer->expire_time)
 	{
-	    LOG(1, 0, 0, 0, "No message from BGP peer %s in %ds\n",
+	    LOG(1, 0, 0, "No message from BGP peer %s in %ds\n",
 		peer->name, peer->hold);
 
 	    bgp_send_notification(peer, BGP_ERR_HOLD_TIMER_EXP, 0);
@@ -605,7 +607,7 @@ int bgp_process(struct bgp_peer *peer, int readable, int writable)
     }
     else if (time_now > peer->state_time + BGP_STATE_TIME)
     {
-	LOG(1, 0, 0, 0, "%s timer expired for BGP peer %s\n",
+	LOG(1, 0, 0, "%s timer expired for BGP peer %s\n",
 	    bgp_state_str(peer->state), peer->name);
 
 	return bgp_restart(peer);
@@ -641,7 +643,7 @@ char const *bgp_state_str(enum bgp_state state)
     return "?";
 }
 
-static char const *bgp_msg_type_str(u8 type)
+static char const *bgp_msg_type_str(uint8_t type)
 {
     switch (type)
     {
@@ -665,7 +667,7 @@ static int bgp_connect(struct bgp_peer *peer)
 	struct servent *serv;
 	if (!(serv = getservbyname("bgp", "tcp")))
 	{
-	    LOG(0, 0, 0, 0, "Can't get bgp service (%s)\n", strerror(errno));
+	    LOG(0, 0, 0, "Can't get bgp service (%s)\n", strerror(errno));
 	    return 0;
 	}
 
@@ -674,7 +676,7 @@ static int bgp_connect(struct bgp_peer *peer)
 
     if ((peer->sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
     {
-	LOG(0, 0, 0, 0, "Can't create a socket for BGP peer %s (%s)\n",
+	LOG(0, 0, 0, "Can't create a socket for BGP peer %s (%s)\n",
 	    peer->name, strerror(errno));
 
 	peer->state = peer->next_state = Disabled;
@@ -697,7 +699,7 @@ static int bgp_connect(struct bgp_peer *peer)
 
 	if (errno != EINPROGRESS)
 	{
-	    LOG(1, 0, 0, 0, "Can't connect to BGP peer %s (%s)\n",
+	    LOG(1, 0, 0, "Can't connect to BGP peer %s (%s)\n",
 		inet_ntoa(addr.sin_addr), strerror(errno));
 
 	    bgp_set_retry(peer);
@@ -707,7 +709,7 @@ static int bgp_connect(struct bgp_peer *peer)
 	peer->state = Connect;
 	peer->state_time = time_now;
 
-	LOG(4, 0, 0, 0, "BGP peer %s: state Connect\n", peer->name);
+	LOG(4, 0, 0, "BGP peer %s: state Connect\n", peer->name);
 	return 1;
     }
 
@@ -715,7 +717,7 @@ static int bgp_connect(struct bgp_peer *peer)
     peer->state_time = time_now;
     peer->retry_time = peer->retry_count = 0;
 
-    LOG(4, 0, 0, 0, "BGP peer %s: state Active\n", inet_ntoa(addr.sin_addr));
+    LOG(4, 0, 0, "BGP peer %s: state Active\n", inet_ntoa(addr.sin_addr));
 
     return bgp_send_open(peer);
 }
@@ -728,7 +730,7 @@ static int bgp_handle_connect(struct bgp_peer *peer)
     getsockopt(peer->sock, SOL_SOCKET, SO_ERROR, &err, &len);
     if (err)
     {
-	LOG(1, 0, 0, 0, "Can't connect to BGP peer %s (%s)\n", peer->name,
+	LOG(1, 0, 0, "Can't connect to BGP peer %s (%s)\n", peer->name,
 	    strerror(err));
 
 	bgp_set_retry(peer);
@@ -738,7 +740,7 @@ static int bgp_handle_connect(struct bgp_peer *peer)
     peer->state = Active;
     peer->state_time = time_now;
 
-    LOG(4, 0, 0, 0, "BGP peer %s: state Active\n", peer->name);
+    LOG(4, 0, 0, "BGP peer %s: state Active\n", peer->name);
 
     return bgp_send_open(peer);
 }
@@ -759,9 +761,9 @@ static int bgp_write(struct bgp_peer *peer)
 	    return 1;
 
 	if (errno == EPIPE)
-	    LOG(1, 0, 0, 0, "Connection to BGP peer %s closed\n", peer->name);
+	    LOG(1, 0, 0, "Connection to BGP peer %s closed\n", peer->name);
 	else
-	    LOG(1, 0, 0, 0, "Can't write to BGP peer %s (%s)\n", peer->name,
+	    LOG(1, 0, 0, "Can't write to BGP peer %s (%s)\n", peer->name,
 		strerror(errno));
 
 	bgp_set_retry(peer);
@@ -774,7 +776,7 @@ static int bgp_write(struct bgp_peer *peer)
 	return 1;
     }
 
-    LOG(4, 0, 0, 0, "Sent %s to BGP peer %s\n",
+    LOG(4, 0, 0, "Sent %s to BGP peer %s\n",
 	bgp_msg_type_str(peer->outbuf->packet.header.type), peer->name);
 
     peer->outbuf->packet.header.len = 0;
@@ -794,7 +796,7 @@ static int bgp_write(struct bgp_peer *peer)
 	peer->state = peer->next_state;
 	peer->state_time = time_now;
 
-	LOG(4, 0, 0, 0, "BGP peer %s: state %s\n", peer->name,
+	LOG(4, 0, 0, "BGP peer %s: state %s\n", peer->name,
 	    bgp_state_str(peer->state));
     }
 
@@ -811,7 +813,7 @@ static int bgp_read(struct bgp_peer *peer)
     {
 	if (!r)
 	{
-	    LOG(1, 0, 0, 0, "Connection to BGP peer %s closed\n", peer->name);
+	    LOG(1, 0, 0, "Connection to BGP peer %s closed\n", peer->name);
 	}
 	else
 	{
@@ -821,7 +823,7 @@ static int bgp_read(struct bgp_peer *peer)
 	    if (errno == EAGAIN)
 		return 1;
 
-	    LOG(1, 0, 0, 0, "Can't read from BGP peer %s (%s)\n", peer->name,
+	    LOG(1, 0, 0, "Can't read from BGP peer %s (%s)\n", peer->name,
 		strerror(errno));
 	}
 
@@ -841,7 +843,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 
     if (len > BGP_MAX_PACKET_SIZE)
     {
-	LOG(1, 0, 0, 0, "Bad header length from BGP %s\n", peer->name);
+	LOG(1, 0, 0, "Bad header length from BGP %s\n", peer->name);
 	bgp_send_notification(peer, BGP_ERR_HEADER, BGP_ERR_HDR_BAD_LEN);
 	return 0;
     }
@@ -849,7 +851,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
     if (peer->inbuf->done < len)
 	return 0;
 
-    LOG(4, 0, 0, 0, "Received %s from BGP peer %s\n",
+    LOG(4, 0, 0, "Received %s from BGP peer %s\n",
 	bgp_msg_type_str(p->header.type), peer->name);
 
     switch (p->header.type)
@@ -864,7 +866,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 	    {
 		if ((unsigned char) p->header.marker[i] != 0xff)
 		{
-		    LOG(1, 0, 0, 0, "Invalid marker from BGP peer %s\n",
+		    LOG(1, 0, 0, "Invalid marker from BGP peer %s\n",
 			peer->name);
 
 		    bgp_send_notification(peer, BGP_ERR_HEADER,
@@ -876,7 +878,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 
 	    if (peer->state != OpenSent)
 	    {
-		LOG(1, 0, 0, 0, "OPEN from BGP peer %s in %s state\n",
+		LOG(1, 0, 0, "OPEN from BGP peer %s in %s state\n",
 		    peer->name, bgp_state_str(peer->state));
 
 		bgp_send_notification(peer, BGP_ERR_FSM, 0);
@@ -887,7 +889,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 
 	    if (data.version != BGP_VERSION)
 	    {
-		LOG(1, 0, 0, 0, "Bad version (%d) sent by BGP peer %s\n",
+		LOG(1, 0, 0, "Bad version (%d) sent by BGP peer %s\n",
 		    (int) data.version, peer->name);
 
 		bgp_send_notification(peer, BGP_ERR_OPEN, BGP_ERR_OPN_VERSION);
@@ -896,7 +898,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 
 	    if (ntohs(data.as) != peer->as)
 	    {
-		LOG(1, 0, 0, 0, "Bad AS sent by BGP peer %s (got %d, "
+		LOG(1, 0, 0, "Bad AS sent by BGP peer %s (got %d, "
 		    "expected %d)\n", peer->name, (int) htons(data.as),
 		    (int) peer->as);
 
@@ -906,7 +908,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 
 	    if ((hold = ntohs(data.hold_time)) < 3)
 	    {
-		LOG(1, 0, 0, 0, "Bad hold time (%d) from BGP peer %s\n",
+		LOG(1, 0, 0, "Bad hold time (%d) from BGP peer %s\n",
 		    hold, peer->name);
 
 		bgp_send_notification(peer, BGP_ERR_OPEN, BGP_ERR_OPN_HOLD_TIME);
@@ -939,7 +941,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 	    peer->retry_count = 0;
 	    peer->retry_time = 0;
 
-	    LOG(4, 0, 0, 0, "BGP peer %s: state Established\n", peer->name);
+	    LOG(4, 0, 0, "BGP peer %s: state Established\n", peer->name);
 	}
 
 	break;
@@ -952,13 +954,13 @@ static int bgp_handle_input(struct bgp_peer *peer)
 
 	    if (notification->error_code == BGP_ERR_CEASE)
 	    {
-		LOG(4, 0, 0, 0, "BGP peer %s sent CEASE\n", peer->name);
+		LOG(4, 0, 0, "BGP peer %s sent CEASE\n", peer->name);
 		bgp_halt(peer);
 		return 0;
 	    }
 
 	    /* FIXME: should handle more notifications */
-	    LOG(4, 0, 0, 0, "BGP peer %s sent unhandled NOTIFICATION %d\n",
+	    LOG(4, 0, 0, "BGP peer %s sent unhandled NOTIFICATION %d\n",
 		peer->name, (int) notification->error_code);
 	}
 
@@ -987,7 +989,7 @@ static int bgp_handle_input(struct bgp_peer *peer)
 static int bgp_send_open(struct bgp_peer *peer)
 {
     struct bgp_data_open data;
-    u16 len = sizeof(peer->outbuf->packet.header);
+    uint16_t len = sizeof(peer->outbuf->packet.header);
 
     memset(peer->outbuf->packet.header.marker, 0xff,
 	sizeof(peer->outbuf->packet.header.marker));
@@ -1029,9 +1031,9 @@ static int bgp_send_keepalive(struct bgp_peer *peer)
 /* send/buffer UPDATE message */
 static int bgp_send_update(struct bgp_peer *peer)
 {
-    u16 unf_len = 0;
-    u16 attr_len;
-    u16 len = sizeof(peer->outbuf->packet.header);
+    uint16_t unf_len = 0;
+    uint16_t attr_len;
+    uint16_t len = sizeof(peer->outbuf->packet.header);
     struct bgp_route_list *have = peer->routes;
     struct bgp_route_list *want = peer->routing ? bgp_routes : 0;
     struct bgp_route_list *e = 0;
@@ -1077,8 +1079,8 @@ static int bgp_send_update(struct bgp_peer *peer)
 	    unf_len += s;
 	    len += s;
 
-	    LOG(5, 0, 0, 0, "Withdrawing route %s/%d from BGP peer %s\n",
-		inet_toa(tmp->dest.prefix), tmp->dest.len, peer->name);
+	    LOG(5, 0, 0, "Withdrawing route %s/%d from BGP peer %s\n",
+		fmtaddr(tmp->dest.prefix, 0), tmp->dest.len, peer->name);
 
 	    free(tmp);
 
@@ -1127,8 +1129,8 @@ static int bgp_send_update(struct bgp_peer *peer)
     {
 	if (!(e = malloc(sizeof(*e))))
 	{
-	    LOG(0, 0, 0, 0, "Can't allocate route for %s/%d (%s)\n",
-		inet_toa(add->dest.prefix), add->dest.len, strerror(errno));
+	    LOG(0, 0, 0, "Can't allocate route for %s/%d (%s)\n",
+		fmtaddr(add->dest.prefix, 0), add->dest.len, strerror(errno));
 
 	    return 0;
 	}
@@ -1151,8 +1153,8 @@ static int bgp_send_update(struct bgp_peer *peer)
 	data += s;
 	len += s;
 
-	LOG(5, 0, 0, 0, "Advertising route %s/%d to BGP peer %s\n",
-	    inet_toa(add->dest.prefix), add->dest.len, peer->name);
+	LOG(5, 0, 0, "Advertising route %s/%d to BGP peer %s\n",
+	    fmtaddr(add->dest.prefix, 0), add->dest.len, peer->name);
     }
     else
     {
@@ -1169,10 +1171,11 @@ static int bgp_send_update(struct bgp_peer *peer)
 }
 
 /* send/buffer NOTIFICATION message */
-static int bgp_send_notification(struct bgp_peer *peer, u8 code, u8 subcode)
+static int bgp_send_notification(struct bgp_peer *peer, uint8_t code,
+    uint8_t subcode)
 {
     struct bgp_data_notification data;
-    u16 len = 0;
+    uint16_t len = 0;
 
     data.error_code = code;
     len += sizeof(data.error_code);
