@@ -1,5 +1,5 @@
 // L2TPNS Global Stuff
-// $Id: l2tpns.h,v 1.35 2004/11/16 07:54:32 bodea Exp $
+// $Id: l2tpns.h,v 1.49.2.1 2005/01/06 01:39:23 bodea Exp $
 
 #ifndef __L2TPNS_H__
 #define __L2TPNS_H__
@@ -15,7 +15,7 @@
 #include <sys/types.h>
 #include <libcli.h>
 
-#define VERSION	"2.0.8"
+#define VERSION	"2.0.14"
 
 // Limits
 #define MAXTUNNEL	500		// could be up to 65535
@@ -77,6 +77,7 @@
 #define CLIUSERS	FLASHDIR "/users"		// CLI Users file
 #define IPPOOLFILE	FLASHDIR "/ip_pool"		// Address pool configuration
 #define ACCT_TIME	3000		// 5 minute accounting interval
+#define ACCT_SHUT_TIME	600		// 1 minute for counters of shutdown sessions
 #define	L2TPPORT	1701		// L2TP port
 #define RADPORT		1645		// old radius port...
 #define	PKTARP		0x0806		// ARP packet type
@@ -107,23 +108,20 @@ enum
 };
 
 // Types
-typedef unsigned short u16;
-typedef unsigned int u32;
-typedef unsigned char u8;
-typedef u32 ipt;
-typedef u16 portt;
-typedef u16 sessionidt;
-typedef u16 tunnelidt;
-typedef u32 clockt;
-typedef u8 hasht[16];
+typedef uint16_t sessionidt;
+typedef uint16_t tunnelidt;
+typedef uint32_t clockt;
+typedef uint8_t hasht[16];
 
 // CLI actions
 struct cli_session_actions {
 	char action;
-	ipt snoop_ip;
-	u16 snoop_port;
+	in_addr_t snoop_ip;
+	uint16_t snoop_port;
 	int throttle_in;
 	int throttle_out;
+	int filter_in;
+	int filter_out;
 };
 
 #define CLI_SESS_KILL		0x01
@@ -131,6 +129,8 @@ struct cli_session_actions {
 #define CLI_SESS_NOSNOOP	0x04
 #define CLI_SESS_THROTTLE	0x08
 #define CLI_SESS_NOTHROTTLE	0x10
+#define CLI_SESS_FILTER		0x20
+#define CLI_SESS_NOFILTER	0x40
 
 struct cli_tunnel_actions {
 	char action;
@@ -142,87 +142,91 @@ struct cli_tunnel_actions {
 #define DUMP_MAGIC "L2TPNS#" VERSION "#"
 
 // structures
-typedef struct routes           // route
+typedef struct			// route
 {
-	ipt ip;
-	ipt mask;
+	in_addr_t ip;
+	in_addr_t mask;
 }
 routet;
 
-typedef struct controls         // control message
+typedef struct controls		// control message
 {
-	struct controls *next;  // next in queue
-	u16 length;             // length
-	u8 buf[MAXCONTROL];
+	struct controls *next;	// next in queue
+	uint16_t length;	// length
+	uint8_t buf[MAXCONTROL];
 }
 controlt;
 
-typedef struct sessions
+typedef struct
 {
 	sessionidt next;		// next session in linked list
 	sessionidt far;			// far end session ID
 	tunnelidt tunnel;		// near end tunnel ID
-	ipt ip;				// IP of session set by RADIUS response (host byte order).
+	in_addr_t ip;			// IP of session set by RADIUS response (host byte order).
 	int ip_pool_index;		// index to IP pool
 	unsigned long unique_id;	// unique session id
-	u16 nr;				// next receive
-	u16 ns;				// next send
-	u32 magic;			// ppp magic number
-	u32 cin, cout;			// byte counts
-	u32 pin, pout;			// packet counts
-	u32 total_cin;			// This counter is never reset while a session is open
-	u32 total_cout;			// This counter is never reset while a session is open
-	u32 id;				// session id
-	u16 throttle_in;		// upstream throttle rate (kbps)
-	u16 throttle_out;		// downstream throttle rate
+	uint16_t nr;			// next receive
+	uint16_t ns;			// next send
+	uint32_t magic;			// ppp magic number
+	uint32_t cin, cout;		// byte counts
+	uint32_t pin, pout;		// packet counts
+	uint32_t total_cin;		// This counter is never reset while a session is open
+	uint32_t total_cout;		// This counter is never reset while a session is open
+	uint32_t id;			// session id
+	uint16_t throttle_in;		// upstream throttle rate (kbps)
+	uint16_t throttle_out;		// downstream throttle rate
 	clockt opened;			// when started
 	clockt die;			// being closed, when to finally free
 	time_t last_packet;		// Last packet from the user (used for idle timeouts)
-	ipt dns1, dns2;			// DNS servers
+	in_addr_t dns1, dns2;		// DNS servers
 	routet route[MAXROUTE];		// static routes
-	u16 radius;			// which radius session is being used (0 for not waiting on authentication)
-	u16 mru;			// maximum receive unit
-	u16 tbf_in;			// filter bucket for throttling in from the user.
-	u16 tbf_out;			// filter bucket for throttling out to the user.
-	u8 l2tp_flags;			// various bit flags from the ICCN on the l2tp tunnel.
-	u8 reserved_old_snoop;		// No longer used - remove at some time
-	u8 walled_garden;		// is this session gardened?
-	u8 flags1;			// additional flags (currently unused);
+	uint16_t radius;		// which radius session is being used (0 for not waiting on authentication)
+	uint16_t mru;			// maximum receive unit
+	uint16_t tbf_in;		// filter bucket for throttling in from the user.
+	uint16_t tbf_out;		// filter bucket for throttling out to the user.
+	uint8_t l2tp_flags;		// various bit flags from the ICCN on the l2tp tunnel.
+	uint8_t reserved_old_snoop;	// No longer used - remove at some time
+	uint8_t walled_garden;		// is this session gardened?
+	uint8_t flags1;			// additional flags (currently unused);
 	char random_vector[MAXTEL];
 	int random_vector_length;
 	char user[129];			// user (needed in seesion for radius stop messages) (can we reduce this? --mo)
 	char called[MAXTEL];		// called number
 	char calling[MAXTEL];		// calling number
-	u32 tx_connect_speed;
-	u32 rx_connect_speed;
-	u32 flags;			// Various session flags.
-	ipt snoop_ip;			// Interception destination IP
-	u16 snoop_port;			// Interception destination port
-	u16 sid;			// near end session id.
-	char reserved[20];		// Space to expand structure without changing HB_VERSION
+	uint32_t tx_connect_speed;
+	uint32_t rx_connect_speed;
+	uint32_t flags;			// Various session flags.
+	in_addr_t snoop_ip;		// Interception destination IP
+	uint16_t snoop_port;		// Interception destination port
+	uint16_t sid;			// near end session id.
+	uint8_t filter_in;		// input filter index (to ip_filters[N-1]; 0 if none)
+	uint8_t filter_out;		// output filter index
+	char reserved[18];		// Space to expand structure without changing HB_VERSION
 }
 sessiont;
 
 #define SF_IPCP_ACKED	1	// Has this session seen an IPCP Ack?
 #define SF_LCP_ACKED	2	// LCP negotiated
+#define SF_CCP_ACKED	4	// CCP negotiated
 
-typedef struct {
-	u32	cin;
-	u32	cout;
+typedef struct
+{
+	uint32_t cin;
+	uint32_t cout;
 } sessioncountt;
 
 #define	SESSIONPFC	1	// PFC negotiated flags
 #define	SESSIONACFC	2	// ACFC negotiated flags
 
 // 168 bytes per tunnel
-typedef struct tunnels
+typedef struct
 {
 	tunnelidt far;		// far end tunnel ID
-	ipt ip;			// Ip for far end
-	portt port;		// port for far end
-	u16 window;		// Rx window
-	u16 nr;			// next receive
-	u16 ns;			// next send
+	in_addr_t ip;		// Ip for far end
+	uint16_t port;		// port for far end
+	uint16_t window;	// Rx window
+	uint16_t nr;		// next receive
+	uint16_t ns;		// next send
 	int state;		// current state (tunnelstate enum)
 	clockt last;		// when last control message sent (used for resend timeout)
 	clockt retry;		// when to try resenting pending control
@@ -230,31 +234,31 @@ typedef struct tunnels
 	clockt lastrec;		// when the last control message was received
 	char hostname[128];	// tunnel hostname
 	char vendor[128];	// LAC vendor
-	u8 try;			// number of retrys on a control message
-	u16 controlc;		// outstaind messages in queue
+	uint8_t try;		// number of retrys on a control message
+	uint16_t controlc;	// outstaind messages in queue
 	controlt *controls;	// oldest message
 	controlt *controle;	// newest message
 }
 tunnelt;
 
 // 180 bytes per radius session
-typedef struct radiuss		// outstanding RADIUS requests
+typedef struct			// outstanding RADIUS requests
 {
 	sessionidt session;	// which session this applies to
 	hasht auth;		// request authenticator
 	clockt retry;		// when to try next
 	char calling[MAXTEL];	// calling number
 	char pass[129];		// password
-	u8 id;			// ID for PPP response
-	u8 try;			// which try we are on
-	u8 state;		// state of radius requests
-	u8 chap;		// set if CHAP used (is CHAP identifier)
+	uint8_t id;		// ID for PPP response
+	uint8_t try;		// which try we are on
+	uint8_t state;		// state of radius requests
+	uint8_t chap;		// set if CHAP used (is CHAP identifier)
 }
 radiust;
 
 typedef struct
 {
-	ipt		address;	// Host byte order..
+	in_addr_t	address;	// Host byte order..
 	char		assigned;	// 1 if assigned, 0 if free
 	sessionidt	session;
 	clockt		last;		// last used
@@ -269,7 +273,6 @@ struct Tringbuffer
 		char level;
 		sessionidt session;
 		tunnelidt tunnel;
-		ipt address;
 		char message[MAX_LOG_LENGTH];
 	} buffer[RINGBUFFER_SIZE];
 	int head;
@@ -304,71 +307,76 @@ enum
 
 struct Tstats
 {
-    time_t		start_time;
-    time_t		last_reset;
+    time_t	start_time;
+    time_t	last_reset;
 
-    unsigned long	tun_rx_packets;
-    unsigned long	tun_tx_packets;
-    unsigned long	tun_rx_bytes;
-    unsigned long	tun_tx_bytes;
-    unsigned long	tun_rx_errors;
-    unsigned long	tun_tx_errors;
+    uint32_t	tun_rx_packets;
+    uint32_t	tun_tx_packets;
+    uint32_t	tun_rx_bytes;
+    uint32_t	tun_tx_bytes;
+    uint32_t	tun_rx_errors;
+    uint32_t	tun_tx_errors;
 
-    unsigned long	tunnel_rx_packets;
-    unsigned long	tunnel_tx_packets;
-    unsigned long	tunnel_rx_bytes;
-    unsigned long	tunnel_tx_bytes;
-    unsigned long	tunnel_rx_errors;
-    unsigned long	tunnel_tx_errors;
+    uint32_t	tunnel_rx_packets;
+    uint32_t	tunnel_tx_packets;
+    uint32_t	tunnel_rx_bytes;
+    uint32_t	tunnel_tx_bytes;
+    uint32_t	tunnel_rx_errors;
+    uint32_t	tunnel_tx_errors;
 
-    unsigned long	tunnel_retries;
-    unsigned long	radius_retries;
+    uint32_t	tunnel_retries;
+    uint32_t	radius_retries;
 
-    unsigned long	arp_sent;
+    uint32_t	arp_sent;
 
-    unsigned long	packets_snooped;
+    uint32_t	packets_snooped;
 
-    unsigned long	tunnel_created;
-    unsigned long	session_created;
-    unsigned long	tunnel_timeout;
-    unsigned long	session_timeout;
-    unsigned long	radius_timeout;
-    unsigned long	radius_overflow;
-    unsigned long	tunnel_overflow;
-    unsigned long	session_overflow;
+    uint32_t	tunnel_created;
+    uint32_t	session_created;
+    uint32_t	tunnel_timeout;
+    uint32_t	session_timeout;
+    uint32_t	radius_timeout;
+    uint32_t	radius_overflow;
+    uint32_t	tunnel_overflow;
+    uint32_t	session_overflow;
 
-    unsigned long	ip_allocated;
-    unsigned long	ip_freed;
+    uint32_t	ip_allocated;
+    uint32_t	ip_freed;
+
+    uint32_t	c_forwarded;
+    uint32_t	recv_forward;
+
+    uint32_t	select_called;
+    uint32_t	multi_read_used;
+    uint32_t	multi_read_exceeded;
 
-    unsigned long	c_forwarded;
-    unsigned long	recv_forward;
 #ifdef STATISTICS
-    unsigned long	call_processtun;
-    unsigned long	call_processipout;
-    unsigned long	call_processudp;
-    unsigned long	call_sessionbyip;
-    unsigned long	call_sessionbyuser;
-    unsigned long	call_sendarp;
-    unsigned long	call_sendipcp;
-    unsigned long	call_tunnelsend;
-    unsigned long	call_sessionkill;
-    unsigned long	call_sessionshutdown;
-    unsigned long	call_tunnelkill;
-    unsigned long	call_tunnelshutdown;
-    unsigned long	call_assign_ip_address;
-    unsigned long	call_free_ip_address;
-    unsigned long	call_dump_acct_info;
-    unsigned long	call_sessionsetup;
-    unsigned long	call_processpap;
-    unsigned long	call_processchap;
-    unsigned long	call_processlcp;
-    unsigned long	call_processipcp;
-    unsigned long	call_processipin;
-    unsigned long	call_processccp;
-    unsigned long	call_sendchap;
-    unsigned long	call_processrad;
-    unsigned long	call_radiussend;
-    unsigned long	call_radiusretry;
+    uint32_t	call_processtun;
+    uint32_t	call_processipout;
+    uint32_t	call_processudp;
+    uint32_t	call_sessionbyip;
+    uint32_t	call_sessionbyuser;
+    uint32_t	call_sendarp;
+    uint32_t	call_sendipcp;
+    uint32_t	call_tunnelsend;
+    uint32_t	call_sessionkill;
+    uint32_t	call_sessionshutdown;
+    uint32_t	call_tunnelkill;
+    uint32_t	call_tunnelshutdown;
+    uint32_t	call_assign_ip_address;
+    uint32_t	call_free_ip_address;
+    uint32_t	call_dump_acct_info;
+    uint32_t	call_sessionsetup;
+    uint32_t	call_processpap;
+    uint32_t	call_processchap;
+    uint32_t	call_processlcp;
+    uint32_t	call_processipcp;
+    uint32_t	call_processipin;
+    uint32_t	call_processccp;
+    uint32_t	call_sendchap;
+    uint32_t	call_processrad;
+    uint32_t	call_radiussend;
+    uint32_t	call_radiusretry;
 #endif
 };
 
@@ -392,7 +400,7 @@ struct Tstats
 #define SET_STAT(x, y)
 #endif
 
-struct configt
+typedef struct
 {
 	int		debug;				// debugging level
 	time_t		start_time;			// time when l2tpns was started
@@ -414,20 +422,20 @@ struct configt
 
 	char		radiussecret[64];
 	int		radius_accounting;
-	ipt		radiusserver[MAXRADSERVER];	// radius servers
-	u16		radiusport[MAXRADSERVER];	// radius base ports
-	u8		numradiusservers;		// radius server count
+	in_addr_t	radiusserver[MAXRADSERVER];	// radius servers
+	uint16_t	radiusport[MAXRADSERVER];	// radius base ports
+	uint8_t		numradiusservers;		// radius server count
 	short		num_radfds;			// Number of radius filehandles allocated
 
-	ipt		default_dns1, default_dns2;
+	in_addr_t	default_dns1, default_dns2;
 
 	unsigned long	rl_rate;			// default throttle rate
 	int		num_tbfs;			// number of throttle buckets
 
 	int		save_state;
 	char		accounting_dir[128];
-	ipt		bind_address;
-	ipt		peer_address;
+	in_addr_t	bind_address;
+	in_addr_t	peer_address;
 	int		send_garp;			// Set to true to garp for vip address on startup
 
 	int		target_uid;
@@ -440,12 +448,12 @@ struct configt
 	int		lock_pages;			// Lock pages into memory.
 	int		icmp_rate;			// Max number of ICMP unreachable per second to send>
 
-	u32		cluster_address;		// Multicast address of cluster.
+	in_addr_t	cluster_address;		// Multicast address of cluster.
 							// Send to this address to have everyone hear.
 	char		cluster_interface[64];		// Which interface to listen for multicast on.
 	int		cluster_iam_master;		// Are we the cluster master???
 	int		cluster_iam_uptodate;		// Set if we've got a full set of state from the master.
-	u32		cluster_master_address;		// The network address of the cluster master.
+	in_addr_t	cluster_master_address;		// The network address of the cluster master.
 							// Zero if i am the cluster master.
 	int		cluster_seq_number;		// Sequence number of the next heartbeat we'll send out
 							// (or the seq number we're next expecting if we're a slave).
@@ -458,74 +466,128 @@ struct configt
 
 	int		cluster_hb_interval;		// How often to send a heartbeat.
 	int		cluster_hb_timeout;		// How many missed heartbeats trigger an election.
+	uint64_t	cluster_table_version;		// # state changes processed by cluster
 
 #ifdef BGP
 #define BGP_NUM_PEERS	2
-	u16		as_number;
+	uint16_t as_number;
 	struct {
-		char	name[64];
-	    	u16	as;
-		int	keepalive;
-		int	hold;
-	}		neighbour[BGP_NUM_PEERS];
+		char name[64];
+	    	uint16_t as;
+		int keepalive;
+		int hold;
+	} neighbour[BGP_NUM_PEERS];
 #endif
-};
+} configt;
 
 enum config_typet { INT, STRING, UNSIGNED_LONG, SHORT, BOOL, IP, MAC };
-struct config_descriptt
+typedef struct
 {
 	char *key;
 	int offset;
 	int size;
 	enum config_typet type;
-};
+} config_descriptt;
+
+typedef struct
+{
+	uint8_t op;	// operation
+#define FILTER_PORT_OP_NONE	0 // all ports match
+#define FILTER_PORT_OP_EQ	1
+#define FILTER_PORT_OP_NEQ	2
+#define FILTER_PORT_OP_GT	3
+#define FILTER_PORT_OP_LT	4
+#define FILTER_PORT_OP_RANGE	5
+	uint16_t port;	// port (host byte order)
+	uint16_t port2;	// range
+} ip_filter_portt;
+
+typedef struct
+{
+	int action;		// permit/deny
+#define FILTER_ACTION_DENY	1
+#define FILTER_ACTION_PERMIT	2
+	uint8_t proto;		// protocol: IPPROTO_* (netinet/in.h)
+	in_addr_t src_ip;	// source ip (network byte order)
+	in_addr_t src_wild;
+	ip_filter_portt src_ports;
+	in_addr_t dst_ip;	// dest ip
+	in_addr_t dst_wild;
+	ip_filter_portt dst_ports;
+	uint8_t frag;		// apply to non-initial fragments
+	uint8_t tcp_flag_op;	// match type: any, all, established
+#define FILTER_FLAG_OP_ANY	1
+#define FILTER_FLAG_OP_ALL	2
+#define FILTER_FLAG_OP_EST	3
+	uint8_t tcp_sflags;	// flags set
+	uint8_t tcp_cflags;	// flags clear
+	uint32_t counter;	// match count
+} ip_filter_rulet;
+
+#define TCP_FLAG_FIN	0x01
+#define TCP_FLAG_SYN	0x02
+#define TCP_FLAG_RST	0x04
+#define TCP_FLAG_PSH	0x08
+#define TCP_FLAG_ACK	0x10
+#define TCP_FLAG_URG	0x20
+
+#define MAXFILTER		32
+#define MAXFILTER_RULES		32
+typedef struct
+{
+    	char name[32];		// ACL name
+	int extended;		// type: 0 = standard, 1 = extended
+	ip_filter_rulet rules[MAXFILTER_RULES];
+	int used;		// session ref count
+} ip_filtert;
 
 // arp.c
-void sendarp(int ifr_idx, const unsigned char* mac, ipt ip);
+void sendarp(int ifr_idx, const unsigned char* mac, in_addr_t ip);
 
 
 // ppp.c
-void processpap(tunnelidt t, sessionidt s, u8 * p, u16 l);
-void processchap(tunnelidt t, sessionidt s, u8 * p, u16 l);
-void processlcp(tunnelidt t, sessionidt s, u8 * p, u16 l);
-void processipcp(tunnelidt t, sessionidt s, u8 * p, u16 l);
-void processipin(tunnelidt t, sessionidt s, u8 * p, u16 l);
-void processccp(tunnelidt t, sessionidt s, u8 * p, u16 l);
+void processpap(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l);
+void processchap(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l);
+void processlcp(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l);
+void processipcp(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l);
+void processipin(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l);
+void processccp(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l);
 void sendchap(tunnelidt t, sessionidt s);
-u8 *makeppp(u8 * b, int size, u8 * p, int l, tunnelidt t, sessionidt s, u16 mtype);
+uint8_t *makeppp(uint8_t *b, int size, uint8_t *p, int l, tunnelidt t, sessionidt s, uint16_t mtype);
 void initlcp(tunnelidt t, sessionidt s);
-void send_ipin(sessionidt s, u8 * buf, int len);
+void send_ipin(sessionidt s, uint8_t *buf, int len);
 
 
 // radius.c
 void initrad(void);
-void radiussend(u16 r, u8 state);
-void processrad(u8 *buf, int len, char socket_index);
-void radiusretry(u16 r);
-u16 radiusnew(sessionidt s);
-void radiusclear(u16 r, sessionidt s);
+void radiussend(uint16_t r, uint8_t state);
+void processrad(uint8_t *buf, int len, char socket_index);
+void radiusretry(uint16_t r);
+uint16_t radiusnew(sessionidt s);
+void radiusclear(uint16_t r, sessionidt s);
 
 
 // l2tpns.c
-clockt backoff(u8 try);
-sessionidt sessionbyip(ipt ip);
+clockt backoff(uint8_t try);
+sessionidt sessionbyip(in_addr_t ip);
 sessionidt sessionbyuser(char *username);
 void sessionshutdown(sessionidt s, char *reason);
-void send_garp(ipt ip);
-void tunnelsend(u8 * buf, u16 l, tunnelidt t);
+void send_garp(in_addr_t ip);
+void tunnelsend(uint8_t *buf, uint16_t l, tunnelidt t);
 void sendipcp(tunnelidt t, sessionidt s);
-void processudp(u8 * buf, int len, struct sockaddr_in *addr);
-void snoop_send_packet(char *packet, u16 size, ipt destination, u16 port);
+void processudp(uint8_t *buf, int len, struct sockaddr_in *addr);
+void snoop_send_packet(char *packet, uint16_t size, in_addr_t destination, uint16_t port);
+int ip_filter(uint8_t *buf, int len, uint8_t filter);
 int cmd_show_ipcache(struct cli_def *cli, char *command, char **argv, int argc);
 int cmd_show_hist_idle(struct cli_def *cli, char *command, char **argv, int argc);
 int cmd_show_hist_open(struct cli_def *cli, char *command, char **argv, int argc);
 
 #undef LOG
 #undef LOG_HEX
-#define LOG(D, a, s, t, f, ...)	({ if (D <= config->debug) _log(D, a, s, t, f, ## __VA_ARGS__); })
+#define LOG(D, s, t, f, ...)	({ if (D <= config->debug) _log(D, s, t, f, ## __VA_ARGS__); })
 #define LOG_HEX(D, t, d, s)	({ if (D <= config->debug) _log_hex(D, t, d, s); })
 
-void _log(int level, ipt address, sessionidt s, tunnelidt t, const char *format, ...) __attribute__((format (printf, 5, 6)));
+void _log(int level, sessionidt s, tunnelidt t, const char *format, ...) __attribute__((format (printf, 4, 5)));
 void _log_hex(int level, const char *title, const char *data, int maxsize);
 
 int sessionsetup(tunnelidt t, sessionidt s);
@@ -544,7 +606,7 @@ int cli_arg_help(struct cli_def *cli, int cr_ok, char *entry, ...);
 
 
 // icmp.c
-void host_unreachable(ipt destination, u16 id, ipt source, char *packet, int packet_len);
+void host_unreachable(in_addr_t destination, uint16_t id, in_addr_t source, char *packet, int packet_len);
 
 
 extern tunnelt *tunnel;
@@ -558,24 +620,24 @@ if (count++ < max) { \
 	void *array[20]; \
 	char **strings; \
 	int size, i; \
-	LOG(0, 0, 0, t, "Backtrace follows"); \
+	LOG(0, 0, t, "Backtrace follows"); \
 	size = backtrace(array, 10); \
 	strings = backtrace_symbols(array, size); \
 	if (strings) for (i = 0; i < size; i++) \
 	{ \
-		LOG(0, 0, 0, t, "%s\n", strings[i]); \
+		LOG(0, 0, t, "%s\n", strings[i]); \
 	} \
 	free(strings); \
 }
 
 
-extern struct configt *config;
+extern configt *config;
 extern time_t basetime;		// Time when this process started.
 extern time_t time_now;		// Seconds since EPOCH.
-extern u32 last_id;
+extern uint32_t last_id;
 extern struct Tstats *_statistics;
-extern ipt my_address;
-extern int tun_write(u8 *data, int size);
+extern in_addr_t my_address;
+extern int tun_write(uint8_t *data, int size);
 extern int clifd;
 
 
