@@ -1,6 +1,6 @@
 // L2TPNS PPP Stuff
 
-char const *cvs_id_ppp = "$Id: ppp.c,v 1.27 2004/11/18 13:15:28 bodea Exp $";
+char const *cvs_id_ppp = "$Id: ppp.c,v 1.39 2004/12/16 08:49:53 bodea Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -19,32 +19,32 @@ extern sessiont *session;
 extern radiust *radius;
 extern int tunfd;
 extern char hostname[];
-extern u32 eth_tx;
+extern uint32_t eth_tx;
 extern time_t time_now;
-extern struct configt *config;
+extern configt *config;
 
-static void sendccp(tunnelidt t, sessionidt s);
+static void initccp(tunnelidt t, sessionidt s);
 
 // Process PAP messages
-void processpap(tunnelidt t, sessionidt s, u8 *p, u16 l)
+void processpap(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 {
 	char user[129];
 	char pass[129];
-	u16 hl;
+	uint16_t hl;
 
 	CSTAT(call_processpap);
 
 	LOG_HEX(5, "PAP", p, l);
 	if (l < 4)
 	{
-		LOG(1, 0, s, t, "Short PAP %u bytes\n", l);
+		LOG(1, s, t, "Short PAP %u bytes\n", l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 
-	if ((hl = ntohs(*(u16 *) (p + 2))) > l)
+	if ((hl = ntohs(*(uint16_t *) (p + 2))) > l)
 	{
-		LOG(1, 0, s, t, "Length mismatch PAP %u/%u\n", hl, l);
+		LOG(1, s, t, "Length mismatch PAP %u/%u\n", hl, l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -52,13 +52,13 @@ void processpap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 
 	if (*p != 1)
 	{
-		LOG(1, 0, s, t, "Unexpected PAP code %d\n", *p);
+		LOG(1, s, t, "Unexpected PAP code %d\n", *p);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 
 	{
-		u8 *b = p;
+		uint8_t *b = p;
 		b += 4;
 		if (*b && *b < sizeof(user))
 			memcpy(user, b + 1, *b);
@@ -67,48 +67,48 @@ void processpap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		if (*b && *b < sizeof(pass))
 			memcpy(pass, b + 1, *b);
 		pass[*b] = 0;
-		LOG(3, 0, s, t, "PAP login %s/%s\n", user, pass);
+		LOG(3, s, t, "PAP login %s/%s\n", user, pass);
 	}
 	if (session[s].ip || !session[s].radius)
 	{
 		// respond now, either no RADIUS available or already authenticated
-		u8 b[MAXCONTROL];
-		u8 id = p[1];
-		u8 *p = makeppp(b, sizeof(b), 0, 0, t, s, PPPPAP);
-		if (!p) {	// Failed to make ppp header!
-			LOG(1,0,0,0, "Failed to make PPP header in process pap!\n");
-			return;
-		}
+		uint8_t b[MAXCONTROL];
+		uint8_t id = p[1];
+		uint8_t *p = makeppp(b, sizeof(b), 0, 0, t, s, PPPPAP);
+		if (!p) return;
+
 		if (session[s].ip)
-			*p = 2;			// ACK
+			*p = 2;				// ACK
 		else
-			*p = 3;			// cant authorise
+			*p = 3;				// cant authorise
 		p[1] = id;
-		*(u16 *) (p + 2) = htons(5);	// length
-		p[4] = 0;			// no message
+		*(uint16_t *) (p + 2) = htons(5);	// length
+		p[4] = 0;				// no message
 		if (session[s].ip)
 		{
-			LOG(3, session[s].ip, s, t, "Already an IP allocated: %s (%d)\n", inet_toa(htonl(session[s].ip)), session[s].ip_pool_index);
+			LOG(3, s, t, "Already an IP allocated: %s (%d)\n",
+				fmtaddr(htonl(session[s].ip), 0), session[s].ip_pool_index);
+
 			session[s].flags &= ~SF_IPCP_ACKED;
 		}
 		else
 		{
-			LOG(1, 0, s, t, "No radius session available to authenticate session...\n");
+			LOG(1, s, t, "No radius session available to authenticate session...\n");
 		}
-		LOG(3, 0, s, t, "Fallback response to PAP (%s)\n", (session[s].ip) ? "ACK" : "NAK");
+		LOG(3, s, t, "Fallback response to PAP (%s)\n", (session[s].ip) ? "ACK" : "NAK");
 		tunnelsend(b, 5 + (p - b), t); // send it
 	}
 	else
 	{
 		// set up RADIUS request
-		u16 r = session[s].radius;
+		uint16_t r = session[s].radius;
 
 		// Run PRE_AUTH plugins
 		struct param_pre_auth packet = { &tunnel[t], &session[s], strdup(user), strdup(pass), PPPPAP, 1 };
 		run_plugins(PLUGIN_PRE_AUTH, &packet);
 		if (!packet.continue_auth)
 		{
-			LOG(3, 0, s, t, "A plugin rejected PRE_AUTH\n");
+			LOG(3, s, t, "A plugin rejected PRE_AUTH\n");
 			if (packet.username) free(packet.username);
 			if (packet.password) free(packet.password);
 			return;
@@ -121,16 +121,16 @@ void processpap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		free(packet.password);
 
 		radius[r].id = p[1];
-		LOG(3, 0, s, t, "Sending login for %s/%s to radius\n", user, pass);
+		LOG(3, s, t, "Sending login for %s/%s to radius\n", user, pass);
 		radiussend(r, RADIUSAUTH);
 	}
 }
 
 // Process CHAP messages
-void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
+void processchap(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 {
-	u16 r;
-	u16 hl;
+	uint16_t r;
+	uint16_t hl;
 
 	CSTAT(call_processchap);
 
@@ -138,7 +138,7 @@ void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	r = session[s].radius;
 	if (!r)
 	{
-		LOG(1, 0, s, t, "Unexpected CHAP message\n");
+		LOG(1, s, t, "Unexpected CHAP message\n");
 
 // FIXME: Need to drop the session here.
 
@@ -148,14 +148,14 @@ void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 
 	if (l < 4)
 	{
-		LOG(1, 0, s, t, "Short CHAP %u bytes\n", l);
+		LOG(1, s, t, "Short CHAP %u bytes\n", l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 
-	if ((hl = ntohs(*(u16 *) (p + 2))) > l)
+	if ((hl = ntohs(*(uint16_t *) (p + 2))) > l)
 	{
-		LOG(1, 0, s, t, "Length mismatch CHAP %u/%u\n", hl, l);
+		LOG(1, s, t, "Length mismatch CHAP %u/%u\n", hl, l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -163,20 +163,20 @@ void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 
 	if (*p != 2)
 	{
-		LOG(1, 0, s, t, "Unexpected CHAP response code %d\n", *p);
+		LOG(1, s, t, "Unexpected CHAP response code %d\n", *p);
 		STAT(tunnel_rx_errors);
 		return;
 	}
 	if (p[1] != radius[r].id)
 	{
-		LOG(1, 0, s, t, "Wrong CHAP response ID %d (should be %d) (%d)\n", p[1], radius[r].id, r);
+		LOG(1, s, t, "Wrong CHAP response ID %d (should be %d) (%d)\n", p[1], radius[r].id, r);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 
 	if (l < 5 || p[4] != 16)
 	{
-		LOG(1, 0, s, t, "Bad CHAP response length %d\n", l < 5 ? -1 : p[4]);
+		LOG(1, s, t, "Bad CHAP response length %d\n", l < 5 ? -1 : p[4]);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -185,7 +185,7 @@ void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	p += 5;
 	if (l < 16 || l - 16 >= sizeof(session[s].user))
 	{
-		LOG(1, 0, s, t, "CHAP user too long %d\n", l - 16);
+		LOG(1, s, t, "CHAP user too long %d\n", l - 16);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -206,7 +206,7 @@ void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		run_plugins(PLUGIN_PRE_AUTH, &packet);
 		if (!packet.continue_auth)
 		{
-			LOG(3, 0, s, t, "A plugin rejected PRE_AUTH\n");
+			LOG(3, s, t, "A plugin rejected PRE_AUTH\n");
 			if (packet.username) free(packet.username);
 			if (packet.password) free(packet.password);
 			return;
@@ -220,7 +220,7 @@ void processchap(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	}
 
 	radius[r].chap = 1;
-	LOG(3, 0, s, t, "CHAP login %s\n", session[s].user);
+	LOG(3, s, t, "CHAP login %s\n", session[s].user);
 	radiussend(r, RADIUSAUTH);
 }
 
@@ -240,14 +240,14 @@ static char *ppp_lcp_types[] = {
 	"IdentRequest",
 };
 
-static void dumplcp(u8 *p, int l)
+static void dumplcp(uint8_t *p, int l)
 {
 	int x = l - 4;
-	u8 *o = (p + 4);
+	uint8_t *o = (p + 4);
 
 	LOG_HEX(5, "PPP LCP Packet", p, l);
-	LOG(4, 0, 0, 0, "PPP LCP Packet type %d (%s len %d)\n", *p, ppp_lcp_types[(int)*p], ntohs( ((u16 *) p)[1]) );
-	LOG(4, 0, 0, 0, "Length: %d\n", l);
+	LOG(4, 0, 0, "PPP LCP Packet type %d (%s len %d)\n", *p, ppp_lcp_types[(int)*p], ntohs( ((uint16_t *) p)[1]) );
+	LOG(4, 0, 0, "Length: %d\n", l);
 	if (*p != ConfigReq && *p != ConfigRej && *p != ConfigAck)
 		return;
 
@@ -257,12 +257,12 @@ static void dumplcp(u8 *p, int l)
 		int length = o[1];
 		if (length < 2)
 		{
-			LOG(4, 0, 0, 0, "	Option length is %d...\n", length);
+			LOG(4, 0, 0, "	Option length is %d...\n", length);
 			break;
 		}
 		if (type == 0)
 		{
-			LOG(4, 0, 0, 0, "	Option type is 0...\n");
+			LOG(4, 0, 0, "	Option type is 0...\n");
 			x -= length;
 			o += length;
 			continue;
@@ -271,51 +271,51 @@ static void dumplcp(u8 *p, int l)
 		{
 			case 1: // Maximum-Receive-Unit
 				if (length == 4)
-					LOG(4, 0, 0, 0, "    %s %d\n", lcp_types[type], ntohs(*(u16 *)(o + 2)));
+					LOG(4, 0, 0, "    %s %d\n", lcp_types[type], ntohs(*(uint16_t *)(o + 2)));
 				else
-					LOG(4, 0, 0, 0, "    %s odd length %d\n", lcp_types[type], length);
+					LOG(4, 0, 0, "    %s odd length %d\n", lcp_types[type], length);
 				break;
 			case 2: // Async-Control-Character-Map
 				if (length == 6)
 				{
-					u32 asyncmap = ntohl(*(u32 *)(o + 2));
-					LOG(4, 0, 0, 0, "    %s %x\n", lcp_types[type], asyncmap);
+					uint32_t asyncmap = ntohl(*(uint32_t *)(o + 2));
+					LOG(4, 0, 0, "    %s %x\n", lcp_types[type], asyncmap);
 				}
 				else
-					LOG(4, 0, 0, 0, "   %s odd length %d\n", lcp_types[type], length);
+					LOG(4, 0, 0, "   %s odd length %d\n", lcp_types[type], length);
 				break;
 			case 3: // Authentication-Protocol
 				if (length == 4)
 				{
-					int proto = ntohs(*(u16 *)(o + 2));
-					LOG(4, 0, 0, 0, "   %s 0x%x (%s)\n", lcp_types[type], proto,
+					int proto = ntohs(*(uint16_t *)(o + 2));
+					LOG(4, 0, 0, "   %s 0x%x (%s)\n", lcp_types[type], proto,
 						proto == PPPCHAP ? "CHAP" :
 						proto == PPPPAP  ? "PAP"  : "UNKNOWN");
 				}
 				else
-					LOG(4, 0, 0, 0, "   %s odd length %d\n", lcp_types[type], length);
+					LOG(4, 0, 0, "   %s odd length %d\n", lcp_types[type], length);
 				break;
 			case 4: // Quality-Protocol
 				{
-					u32 qp = ntohl(*(u32 *)(o + 2));
-					LOG(4, 0, 0, 0, "    %s %x\n", lcp_types[type], qp);
+					uint32_t qp = ntohl(*(uint32_t *)(o + 2));
+					LOG(4, 0, 0, "    %s %x\n", lcp_types[type], qp);
 				}
 				break;
 			case 5: // Magic-Number
 				if (length == 6)
 				{
-					u32 magicno = ntohl(*(u32 *)(o + 2));
-					LOG(4, 0, 0, 0, "    %s %x\n", lcp_types[type], magicno);
+					uint32_t magicno = ntohl(*(uint32_t *)(o + 2));
+					LOG(4, 0, 0, "    %s %x\n", lcp_types[type], magicno);
 				}
 				else
-					LOG(4, 0, 0, 0, "   %s odd length %d\n", lcp_types[type], length);
+					LOG(4, 0, 0, "   %s odd length %d\n", lcp_types[type], length);
 				break;
 			case 7: // Protocol-Field-Compression
 			case 8: // Address-And-Control-Field-Compression
-				LOG(4, 0, 0, 0, "    %s\n", lcp_types[type]);
+				LOG(4, 0, 0, "    %s\n", lcp_types[type]);
 				break;
 			default:
-				LOG(2, 0, 0, 0, "    Unknown PPP LCP Option type %d\n", type);
+				LOG(2, 0, 0, "    Unknown PPP LCP Option type %d\n", type);
 				break;
 		}
 		x -= length;
@@ -324,26 +324,26 @@ static void dumplcp(u8 *p, int l)
 }
 
 // Process LCP messages
-void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
+void processlcp(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 {
-	u8 b[MAXCONTROL];
-	u8 *q = NULL;
-	u32 magicno = 0;
-	u16 hl;
+	uint8_t b[MAXCONTROL];
+	uint8_t *q = NULL;
+	uint32_t magicno = 0;
+	uint16_t hl;
 
 	CSTAT(call_processlcp);
 
 	LOG_HEX(5, "LCP", p, l);
 	if (l < 4)
 	{
-		LOG(1, session[s].ip, s, t, "Short LCP %d bytes\n", l);
+		LOG(1, s, t, "Short LCP %d bytes\n", l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 
-	if ((hl = ntohs(*(u16 *) (p + 2))) > l)
+	if ((hl = ntohs(*(uint16_t *) (p + 2))) > l)
 	{
-		LOG(1, 0, s, t, "Length mismatch LCP %u/%u\n", hl, l);
+		LOG(1, s, t, "Length mismatch LCP %u/%u\n", hl, l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -351,61 +351,67 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 
 	if (*p == ConfigAck)
 	{
-		LOG(3, session[s].ip, s, t, "LCP: Discarding ConfigAck\n");
+		LOG(3, s, t, "LCP: Discarding ConfigAck\n");
 		session[s].flags |= SF_LCP_ACKED;
 	}
 	else if (*p == ConfigReq)
 	{
 		int x = l - 4;
-		u8 *o = (p + 4);
-		u8 response = 0;
+		uint8_t *o = (p + 4);
+		uint8_t *response = 0;
 
-		LOG(3, session[s].ip, s, t, "LCP: ConfigReq (%d bytes)...\n", l);
+		LOG(3, s, t, "LCP: ConfigReq (%d bytes)...\n", l);
 		if (config->debug > 3) dumplcp(p, l);
 
 		while (x > 2)
 		{
 			int type = o[0];
 			int length = o[1];
+
 			if (length == 0 || type == 0 || x < length) break;
 			switch (type)
 			{
 				case 1: // Maximum-Receive-Unit
-					session[s].mru = ntohs(*(u16 *)(o + 2));
+					session[s].mru = ntohs(*(uint16_t *)(o + 2));
 					break;
 
 				case 2: // Async-Control-Character-Map
-					if (!ntohl(*(u32 *)(o + 2))) // all bits zero is OK
+					if (!ntohl(*(uint32_t *)(o + 2))) // all bits zero is OK
 						break;
 
-					if (response && response != ConfigNak) // rej already queued
+					if (response && *response != ConfigNak) // rej already queued
 						break;
 
-					LOG(2, session[s].ip, s, t, "    Remote requesting asyncmap.  Rejecting.\n");
+					LOG(2, s, t, "    Remote requesting asyncmap.  Rejecting.\n");
 					if (!response)
 					{
-						q = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
-						if (!q)
-						{
-							LOG(2, session[s].ip, s, t, " Failed to send packet.\n");
-							break;
-						}
-						response = *q++ = ConfigNak;
+						q = response = makeppp(b, sizeof(b), p, 2, t, s, PPPLCP);
+						if (!q) break;
+						*q = ConfigNak;
+						q += 4;
 					}
+
+					if ((q - b + 11) > sizeof(b))
+					{
+						LOG(2, s, t, "LCP overflow for asyncmap ConfigNak.\n");
+						break;
+					}
+
 					*q++ = type;
 					*q++ = 6;
 					memset(q, 0, 4); // asyncmap 0
 					q += 4;
+					*((uint16_t *) (response + 2)) = htons(q - response); // LCP header length
 					break;
 
 				case 3: // Authentication-Protocol
 					{
-						int proto = ntohs(*(u16 *)(o + 2));
+						int proto = ntohs(*(uint16_t *)(o + 2));
 						char proto_name[] = "0x0000";
 						if (proto == PPPPAP)
 							break;
 
-						if (response && response != ConfigNak) // rej already queued
+						if (response && *response != ConfigNak) // rej already queued
 							break;
 
 						if (proto == PPPCHAP)
@@ -413,26 +419,31 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 						else
 							sprintf(proto_name, "%#4.4x", proto);
 
-						LOG(2, session[s].ip, s, t, "    Remote requesting %s authentication.  Rejecting.\n", proto_name);
+						LOG(2, s, t, "    Remote requesting %s authentication.  Rejecting.\n", proto_name);
 
 						if (!response)
 						{
-							q = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
-							if (!q)
-							{
-								LOG(2, session[s].ip, s, t, " Failed to send packet.\n");
-								break;
-							}
-							response = *q++ = ConfigNak;
+							q = response = makeppp(b, sizeof(b), p, 2, t, s, PPPLCP);
+							if (!q) break;
+							*q = ConfigNak;
+							q += 4;
 						}
+
+						if ((q - b + length) > sizeof(b))
+						{
+							LOG(2, s, t, "LCP overflow for %s ConfigNak.\n", proto_name);
+							break;
+						}
+
 						memcpy(q, o, length);
-						*(u16 *)(q += 2) = htons(PPPPAP); // NAK -> Use PAP instead
+						*(uint16_t *)(q += 2) = htons(PPPPAP); // NAK -> Use PAP instead
 						q += length;
+						*((uint16_t *) (response + 2)) = htons(q - response);
 					}
 					break;
 
 				case 5: // Magic-Number
-					magicno = ntohl(*(u32 *)(o + 2));
+					magicno = ntohl(*(uint32_t *)(o + 2));
 					break;
 
 				case 4: // Quality-Protocol
@@ -441,20 +452,24 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 					break;
 
 				default: // Reject any unknown options
-					LOG(2, session[s].ip, s, t, "    Rejecting PPP LCP Option type %d\n", type);
-					if (!response || response != ConfigRej) // drop nak in favour of rej
+					LOG(2, s, t, "    Rejecting PPP LCP Option type %d\n", type);
+					if (!response || *response != ConfigRej) // drop nak in favour of rej
 					{
-						q = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
-						if (!q)
-						{
-							LOG(2, session[s].ip, s, t, " Failed to send packet.\n");
-							break;
-						}
-						response = *q++ = ConfigRej;
+						q = response = makeppp(b, sizeof(b), p, 2, t, s, PPPLCP);
+						if (!q) break;
+						*q = ConfigRej;
+						q += 4;
+					}
+
+					if ((q - b + length) > sizeof(b))
+					{
+						LOG(2, s, t, "LCP overflow for ConfigRej (type=%d).\n", type);
+						break;
 					}
 
 					memcpy(q, o, length);
 					q += length;
+					*((uint16_t *) (response + 2)) = htons(q - response); // LCP header length
 			}
 			x -= length;
 			o += length;
@@ -463,16 +478,12 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		if (!response)
 		{
 			// Send back a ConfigAck
-			q = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
-			if (!q)
-			{
-				LOG(3, session[s].ip, s, t, " failed to create packet.\n");
-				return;
-			}
-			response = *q = ConfigAck;
+			q = response = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
+			if (!q) return;
+			*q = ConfigAck;
 		}
 
-		LOG(3, session[s].ip, s, t, "Sending %s\n", ppp_lcp_types[response]);
+		LOG(3, s, t, "Sending %s\n", ppp_lcp_types[*response]);
 		tunnelsend(b, l + (q - b), t);
 
 		if (!(session[s].flags & SF_LCP_ACKED))
@@ -480,22 +491,18 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	}
 	else if (*p == ConfigNak)
 	{
-		LOG(1, session[s].ip, s, t, "Remote end sent a ConfigNak. Ignoring\n");
+		LOG(1, s, t, "Remote end sent a ConfigNak.  Ignoring\n");
 		if (config->debug > 3) dumplcp(p, l);
 		return ;
 	}
 	else if (*p == TerminateReq)
 	{
+		LOG(3, s, t, "LCP: Received TerminateReq.  Sending TerminateAck\n");
 		*p = TerminateAck;	// close
 		q = makeppp(b, sizeof(b),  p, l, t, s, PPPLCP);
-		if (!q)
-		{
-			LOG(3, session[s].ip, s, t, "Failed to create PPP packet in processlcp.\n");
-			return;
-		}
-		LOG(3, session[s].ip, s, t, "LCP: Received TerminateReq. Sending TerminateAck\n");
-		sessionshutdown(s, "Remote end closed connection.");
+		if (!q) return;
 		tunnelsend(b, l + (q - b), t); // send it
+		sessionshutdown(s, "Remote end closed connection.");
 	}
 	else if (*p == TerminateAck)
 	{
@@ -503,15 +510,11 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	}
 	else if (*p == EchoReq)
 	{
+		LOG(5, s, t, "LCP: Received EchoReq.  Sending EchoReply\n");
 		*p = EchoReply;		// reply
-		*(u32 *) (p + 4) = htonl(session[s].magic); // our magic number
+		*(uint32_t *) (p + 4) = htonl(session[s].magic); // our magic number
 		q = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
-		if (!q)
-		{
-			LOG(3, session[s].ip, s, t, " failed to send EchoReply.\n");
-			return;
-		}
-		LOG(5, session[s].ip, s, t, "LCP: Received EchoReq. Sending EchoReply\n");
+		if (!q) return;
 		tunnelsend(b, l + (q - b), t); // send it
 	}
 	else if (*p == EchoReply)
@@ -523,30 +526,26 @@ void processlcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		*p = CodeRej;
 		if (l > MAXCONTROL)
 		{
-			LOG(1, 0, s, t, "Truncated Ident Packet (length=%d) to 1400 bytes\n", l);
+			LOG(1, s, t, "Truncated Ident Packet (length=%d) to 1400 bytes\n", l);
 			l = 1400;
 		}
 		q = makeppp(b, sizeof(b), p, l, t, s, PPPLCP);
-		if (!q)
-		{
-			LOG(3, session[s].ip, s, t, "Failed to create IdentRej.\n");
-			return;
-		}
+		if (!q) return;
 		LOG_HEX(5, "LCPIdentRej", q, l + 4);
 		tunnelsend(b, 12 + 4 + l, t);
 	}
 	else
 	{
-		LOG(1, session[s].ip, s, t, "Unexpected LCP code %d\n", *p);
+		LOG(1, s, t, "Unexpected LCP code %d\n", *p);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 }
 
 // find a PPP option, returns point to option, or 0 if not found
-static u8 *findppp(u8 *b, u8 mtype)
+static uint8_t *findppp(uint8_t *b, uint8_t mtype)
 {
-	u16 l = ntohs(*(u16 *) (b + 2));
+	uint16_t l = ntohs(*(uint16_t *) (b + 2));
 	if (l < 4)
 		return 0;
 	b += 4;
@@ -564,23 +563,23 @@ static u8 *findppp(u8 *b, u8 mtype)
 }
 
 // Process IPCP messages
-void processipcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
+void processipcp(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 {
-	u16 hl;
+	uint16_t hl;
 
 	CSTAT(call_processipcp);
 
 	LOG_HEX(5, "IPCP", p, l);
 	if (l < 5)
 	{
-		LOG(1, 0, s, t, "Short IPCP %d bytes\n", l);
+		LOG(1, s, t, "Short IPCP %d bytes\n", l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
 
-	if ((hl = ntohs(*(u16 *) (p + 2))) > l)
+	if ((hl = ntohs(*(uint16_t *) (p + 2))) > l)
 	{
-		LOG(1, 0, s, t, "Length mismatch IPCP %u/%u\n", hl, l);
+		LOG(1, s, t, "Length mismatch IPCP %u/%u\n", hl, l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -589,7 +588,7 @@ void processipcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	if (*p == ConfigAck)
 	{
 		// happy with our IPCP
-		u16 r = session[s].radius;
+		uint16_t r = session[s].radius;
 		if ((!r || radius[r].state == RADIUSIPCP) && !session[s].walled_garden)
 		{
 			if (!r)
@@ -599,31 +598,30 @@ void processipcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		}
 		session[s].flags |= SF_IPCP_ACKED;
 
-		LOG(3, session[s].ip, s, t, "IPCP Acked, session is now active\n");
+		LOG(3, s, t, "IPCP Acked, session is now active\n");
 
-		// clear  LCP_ACKED flag  for possible fast renegotiaion for routers
-		session[s].flags &= ~SF_LCP_ACKED;
+		// clear LCP_ACKED/CCP_ACKED flag for possible fast renegotiaion for routers
+		session[s].flags &= ~(SF_LCP_ACKED|SF_CCP_ACKED);
 
 		return;
 	}
 	if (*p != ConfigReq)
 	{
-		LOG(1, 0, s, t, "Unexpected IPCP code %d\n", *p);
+		LOG(1, s, t, "Unexpected IPCP code %d\n", *p);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
-	LOG(4, session[s].ip, s, t, "IPCP ConfigReq received\n");
+	LOG(4, s, t, "IPCP ConfigReq received\n");
 
 	if (!session[s].ip)
 	{
-		LOG(3, 0, s, t, "Waiting on radius reply\n");
+		LOG(3, s, t, "Waiting on radius reply\n");
 		return;			// have to wait on RADIUS reply
 	}
 	// form a config reply quoting the IP in the session
 	{
-		u8 b[MAXCONTROL];
-		u8 *i,
-		*q;
+		uint8_t b[MAXCONTROL];
+		uint8_t *i, *q;
 
 		q = p + 4;
 		i = p + l;
@@ -636,70 +634,68 @@ void processipcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 		if (q < i)
 		{
 			// reject
-			u16 n = 4;
+			uint16_t n = 4;
 			i = p + l;
 			if (!(q = makeppp(b, sizeof(b), p, l, t, s, PPPIPCP)))
-			{
-				LOG(2, 0, s, t, "Failed to send IPCP ConfigRej\n");
 				return;
-			}
+
 			*q = ConfigRej;
 			p += 4;
 			while (p < i && p[1])
 			{
 				if (*p != 0x81 && *p != 0x83 && *p != 3)
 				{
-					LOG(2, 0, s, t, "IPCP reject %d\n", *p);
+					LOG(2, s, t, "IPCP reject %d\n", *p);
 					memcpy(q + n, p, p[1]);
 					n += p[1];
 				}
 				p += p[1];
 			}
-			*(u16 *) (q + 2) = htons(n);
-			LOG(4, session[s].ip, s, t, "Sending ConfigRej\n");
+			*(uint16_t *) (q + 2) = htons(n);
+			LOG(4, s, t, "Sending ConfigRej\n");
 			tunnelsend(b, n + (q - b), t); // send it
 		}
 		else
 		{
-			LOG(4, session[s].ip, s, t, "Sending ConfigAck\n");
+			LOG(4, s, t, "Sending ConfigAck\n");
 			*p = ConfigAck;
 			if ((i = findppp(p, 0x81))) // Primary DNS address
 			{
-				if (*(u32 *) (i + 2) != htonl(session[s].dns1))
+				if (*(uint32_t *) (i + 2) != htonl(session[s].dns1))
 				{
-					*(u32 *) (i + 2) = htonl(session[s].dns1);
+					*(uint32_t *) (i + 2) = htonl(session[s].dns1);
 					*p = ConfigNak;
-					LOG(5, session[s].ip, s, t, "   DNS1 = %s\n", inet_toa(session[s].dns1));
+					LOG(5, s, t, "   DNS1 = %s\n",
+						fmtaddr(htonl(session[s].dns1), 0));
 				}
 			}
 			if ((i = findppp(p, 0x83))) // Secondary DNS address (TBA, is it)
 			{
-				if (*(u32 *) (i + 2) != htonl(session[s].dns2))
+				if (*(uint32_t *) (i + 2) != htonl(session[s].dns2))
 				{
-					*(u32 *) (i + 2) = htonl(session[s].dns2);
+					*(uint32_t *) (i + 2) = htonl(session[s].dns2);
 					*p = ConfigNak;
-					LOG(5, session[s].ip, s, t, "   DNS2 = %s\n", inet_toa(session[s].dns2));
+					LOG(5, s, t, "   DNS2 = %s\n",
+						fmtaddr(htonl(session[s].dns2), 0));
 				}
 			}
 			i = findppp(p, 3);		// IP address
 			if (!i || i[1] != 6)
 			{
-				LOG(1, 0, s, t, "No IP in IPCP request\n");
+				LOG(1, s, t, "No IP in IPCP request\n");
 				STAT(tunnel_rx_errors);
 				return ;
 			}
-			if (*(u32 *) (i + 2) != htonl(session[s].ip))
+			if (*(uint32_t *) (i + 2) != htonl(session[s].ip))
 			{
-				*(u32 *) (i + 2) = htonl(session[s].ip);
+				*(uint32_t *) (i + 2) = htonl(session[s].ip);
 				*p = ConfigNak;
-				LOG(4, session[s].ip, s, t, " No, a ConfigNak, client is requesting IP - sending %s\n",
-						inet_toa(htonl(session[s].ip)));
+				LOG(4, s, t, " No, a ConfigNak, client is requesting IP - sending %s\n",
+						fmtaddr(htonl(session[s].ip), 0));
 			}
 			if (!(q = makeppp(b, sizeof(b), p, l, t, s, PPPIPCP)))
-			{
-				LOG(2, 0, s, t, " Failed to send IPCP packet.\n");
 				return;
-			}
+
 			tunnelsend(b, l + (q - b), t); // send it
 		}
 	}
@@ -709,19 +705,19 @@ void processipcp(tunnelidt t, sessionidt s, u8 *p, u16 l)
 //
 // This MUST be called with at least 4 byte behind 'p'.
 // (i.e. this routine writes to p[-4]).
-void processipin(tunnelidt t, sessionidt s, u8 *p, u16 l)
+void processipin(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 {
-	ipt ip;
+	in_addr_t ip;
 
 	CSTAT(call_processipin);
 
 	LOG_HEX(5, "IP", p, l);
 
-	ip = ntohl(*(u32 *)(p + 12));
+	ip = ntohl(*(uint32_t *)(p + 12));
 
 	if (l > MAXETHER)
 	{
-		LOG(1, ip, s, t, "IP packet too long %d\n", l);
+		LOG(1, s, t, "IP packet too long %d\n", l);
 		STAT(tunnel_rx_errors);
 		return ;
 	}
@@ -729,13 +725,17 @@ void processipin(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	// no spoof (do sessionbyip to handled statically routed subnets)
 	if (ip != session[s].ip && sessionbyip(htonl(ip)) != s)
 	{
-		LOG(5, ip, s, t, "Dropping packet with spoofed IP %s\n", inet_toa(htonl(ip)));
+		LOG(5, s, t, "Dropping packet with spoofed IP %s\n", fmtaddr(htonl(ip), 0));
 		return;
 	}
 
+	// run access-list if any
+	if (session[s].filter_in && !ip_filter(p, l, session[s].filter_in - 1))
+		return;
+
 	// Add on the tun header
 	p -= 4;
-	*(u32 *)p = htonl(0x00000800);
+	*(uint32_t *) p = htonl(0x00000800);
 	l += 4;
 
 	if (session[s].tbf_in && !config->cluster_iam_master) { // Are we throttled and a slave?
@@ -752,7 +752,7 @@ void processipin(tunnelidt t, sessionidt s, u8 *p, u16 l)
 	if (tun_write(p, l) < 0)
 	{
 		STAT(tun_tx_errors);
-		LOG(0, 0, s, t, "Error writing %d bytes to TUN device: %s (tunfd=%d, p=%p)\n",
+		LOG(0, s, t, "Error writing %d bytes to TUN device: %s (tunfd=%d, p=%p)\n",
 			l, strerror(errno), tunfd, p);
 
 		return;
@@ -779,14 +779,14 @@ void processipin(tunnelidt t, sessionidt s, u8 *p, u16 l)
 // Helper routine for the TBF filters.
 // Used to send queued data in from the user.
 //
-void send_ipin(sessionidt s, u8 *buf, int len)
+void send_ipin(sessionidt s, uint8_t *buf, int len)
 {
 	LOG_HEX(5, "IP in throttled", buf, len);
 
 	if (write(tunfd, buf, len) < 0)
 	{
 		STAT(tun_tx_errors);
-		LOG(0, 0, 0, 0, "Error writing %d bytes to TUN device: %s (tunfd=%d, p=%p)\n",
+		LOG(0, 0, 0, "Error writing %d bytes to TUN device: %s (tunfd=%d, p=%p)\n",
 			len, strerror(errno), tunfd, buf);
 
 		return;
@@ -812,61 +812,72 @@ void send_ipin(sessionidt s, u8 *buf, int len)
 
 
 // Process CCP messages
-void processccp(tunnelidt t, sessionidt s, u8 *p, u16 l)
+void processccp(tunnelidt t, sessionidt s, uint8_t *p, uint16_t l)
 {
+	uint8_t b[MAXCONTROL];
+	uint8_t *q;
 
 	CSTAT(call_processccp);
 
 	LOG_HEX(5, "CCP", p, l);
-	if (l < 2 || (*p != ConfigReq && *p != TerminateReq))
+	switch (l > 1 ? *p : 0)
 	{
-		LOG(1, 0, s, t, "Unexpected CCP request code %d\n", *p);
-		STAT(tunnel_rx_errors);
-		return ;
-	}
-	// reject
-	{
-		u8 b[MAXCONTROL];
-		u8 *q;
-		if (*p == ConfigReq)
+	case ConfigAck:
+		session[s].flags |= SF_CCP_ACKED;
+		return;
+
+	case ConfigReq:
+		if (l < 6) // accept no compression
 		{
-			if (l < 6)
-			{
-				*p = ConfigAck;		// accept no compression
-			}
-			else
-			{
-				*p = ConfigRej;		// reject
-				sendccp(t, s);
-			}
+			*p = ConfigAck;
+			break;
 		}
+
+		// compression requested--reject
+		*p = ConfigRej;
+
+		// send CCP request for no compression for our end if not negotiated
+		if (!(session[s].flags & SF_CCP_ACKED))
+			initccp(t, s);
+
+		break;
+
+	case TerminateReq:
+	    	*p = TerminateAck;
+		break;
+
+	default:
+		if (l > 1)
+			LOG(1, s, t, "Unexpected CCP request code %d\n", *p);
 		else
-			*p = TerminateAck;		// close
-		if (!(q = makeppp(b, sizeof(b), p, l, t, s, PPPCCP)))
-		{
-			LOG(1,0,0,0, "Failed to send CCP packet.\n");
-			return;
-		}
-		tunnelsend(b, l + (q - b), t); // send it
+			LOG(1, s, t, "Short CCP packet\n");
+
+		STAT(tunnel_rx_errors);
+		return;
 	}
+
+	if (!(q = makeppp(b, sizeof(b), p, l, t, s, PPPCCP)))
+		return;
+
+	tunnelsend(b, l + (q - b), t); // send it
 }
 
 // send a CHAP PP packet
 void sendchap(tunnelidt t, sessionidt s)
 {
-	u8 b[MAXCONTROL];
-	u16 r = session[s].radius;
-	u8 *q;
+	uint8_t b[MAXCONTROL];
+	uint16_t r = session[s].radius;
+	uint8_t *q;
 
 	CSTAT(call_sendchap);
 
 	if (!r)
 	{
-		LOG(1, 0, s, t, "No RADIUS to send challenge\n");
+		LOG(1, s, t, "No RADIUS to send challenge\n");
 		STAT(tunnel_tx_errors);
 		return ;
 	}
-	LOG(1, 0, s, t, "Send CHAP challenge\n");
+	LOG(1, s, t, "Send CHAP challenge\n");
 	{
 		// new challenge
 		int n;
@@ -886,88 +897,97 @@ void sendchap(tunnelidt t, sessionidt s)
 		return ;
 	}
 	q = makeppp(b, sizeof(b), 0, 0, t, s, PPPCHAP);
-	if (!q)
-	{
-		LOG(1, 0, s, t, "failed to send CHAP challenge.\n");
-		return;
-	}
+	if (!q) return;
+
 	*q = 1;					// challenge
 	q[1] = radius[r].id;			// ID
 	q[4] = 16;				// length
 	memcpy(q + 5, radius[r].auth, 16);	// challenge
 	strcpy(q + 21, hostname);		// our name
-	*(u16 *) (q + 2) = htons(strlen(hostname) + 21); // length
+	*(uint16_t *) (q + 2) = htons(strlen(hostname) + 21); // length
 	tunnelsend(b, strlen(hostname) + 21 + (q - b), t); // send it
 }
 
 // fill in a L2TP message with a PPP frame,
 // copies existing PPP message and changes magic number if seen
 // returns start of PPP frame
-u8 *makeppp(u8 *b, int size, u8 *p, int l, tunnelidt t, sessionidt s, u16 mtype)
+uint8_t *makeppp(uint8_t *b, int size, uint8_t *p, int l, tunnelidt t, sessionidt s, uint16_t mtype)
 {
-	if (size < 12)
-		return NULL;	// Need more space than this!!
+	if (size < 12) // Need more space than this!!
+	{
+		static int backtrace_count = 0;
+		LOG(0, s, t, "makeppp buffer too small for L2TP header (size=%d)\n", size);
+		log_backtrace(backtrace_count, 5)
+		return NULL;
+	}
 
-	*(u16 *) (b + 0) = htons(0x0002); // L2TP with no options
-	*(u16 *) (b + 2) = htons(tunnel[t].far); // tunnel
-	*(u16 *) (b + 4) = htons(session[s].far); // session
+	*(uint16_t *) (b + 0) = htons(0x0002); // L2TP with no options
+	*(uint16_t *) (b + 2) = htons(tunnel[t].far); // tunnel
+	*(uint16_t *) (b + 4) = htons(session[s].far); // session
 	b += 6;
 	if (mtype == PPPLCP || !(session[s].l2tp_flags & SESSIONACFC))
 	{
-		*(u16 *) b = htons(0xFF03); // HDLC header
+		*(uint16_t *) b = htons(0xFF03); // HDLC header
 		b += 2;
 	}
 	if (mtype < 0x100 && session[s].l2tp_flags & SESSIONPFC)
 		*b++ = mtype;
 	else
 	{
-		*(u16 *) b = htons(mtype);
+		*(uint16_t *) b = htons(mtype);
 		b += 2;
 	}
 
-	if (l + 12 > size) {
-		LOG(3,0,0,0, "Would have overflowed the buffer in makeppp: size %d, len %d.\n", size, l);
-		return NULL;	// Run out of room to hold the packet!
+	if (l + 12 > size)
+	{
+		static int backtrace_count = 0;
+		LOG(2, s, t, "makeppp would overflow buffer (size=%d, header+payload=%d)\n", size, l + 12);
+		log_backtrace(backtrace_count, 5)
+		return NULL;
 	}
+
 	if (p && l)
 		memcpy(b, p, l);
+
 	return b;
 }
 
-// Send initial LCP ConfigReq
+// Send initial LCP ConfigReq for PAP, set magic no.
 void initlcp(tunnelidt t, sessionidt s)
 {
-	char b[500] = {0}, *q;
+	char b[500], *q;
 
-	q = makeppp(b, sizeof(b), NULL, 0, t, s, PPPLCP);
-	if (!q)
-	{
-		LOG(1, 0, s, t, "Failed to send LCP ConfigReq.\n");
+	if (!(q = makeppp(b, sizeof(b), NULL, 0, t, s, PPPLCP)))
 		return;
-	}
-	LOG(4, 0, s, t, "Sending LCP ConfigReq for PAP\n");
+
+	LOG(4, s, t, "Sending LCP ConfigReq for PAP\n");
 	*q = ConfigReq;
-	*(u8 *)(q + 1) = (time_now % 255) + 1; // ID
-	*(u16 *)(q + 2) = htons(14); // Length
-	*(u8 *)(q + 4) = 5;
-	*(u8 *)(q + 5) = 6;
-	*(u32 *)(q + 6) = htonl(session[s].magic);
-	*(u8 *)(q + 10) = 3;
-	*(u8 *)(q + 11) = 4;
-	*(u16 *)(q + 12) = htons(PPPPAP); // PAP
-	tunnelsend(b, 12 + 14, t);
+	*(uint8_t *)(q + 1) = (time_now % 255) + 1; // ID
+	*(uint16_t *)(q + 2) = htons(14); // Length
+	*(uint8_t *)(q + 4) = 5;
+	*(uint8_t *)(q + 5) = 6;
+	*(uint32_t *)(q + 6) = htonl(session[s].magic);
+	*(uint8_t *)(q + 10) = 3;
+	*(uint8_t *)(q + 11) = 4;
+	*(uint16_t *)(q + 12) = htons(PPPPAP); // PAP
+
+	LOG_HEX(5, "PPPLCP", q, 14);
+	tunnelsend(b, (q - b) + 14, t);
 }
 
-// Send CCP reply
-static void sendccp(tunnelidt t, sessionidt s)
+// Send CCP request for no compression
+static void initccp(tunnelidt t, sessionidt s)
 {
-	char *q, b[500] = {0};
+	char b[500], *q;
 
-	q = makeppp(b, sizeof(b), NULL, 0, t, s, PPPCCP);
+	if (!(q = makeppp(b, sizeof(b), NULL, 0, t, s, PPPCCP)))
+		return;
+
+	LOG(4, s, t, "Sending CCP ConfigReq for no compression\n");
 	*q = ConfigReq;
-	*(u8 *)(q + 1) = (time_now % 255) + 1; // ID
-	*(u16 *)(q + 2) = htons(4); // Length
+	*(uint8_t *)(q + 1) = (time_now % 255) + 1; // ID
+	*(uint16_t *)(q + 2) = htons(4); // Length
+
 	LOG_HEX(5, "PPPCCP", q, 4);
 	tunnelsend(b, (q - b) + 4 , t);
 }
-
