@@ -1,6 +1,6 @@
 // L2TPNS Radius Stuff
 
-char const *cvs_id_radius = "$Id: radius.c,v 1.12 2004/11/16 07:54:32 bodea Exp $";
+char const *cvs_id_radius = "$Id: radius.c,v 1.20 2004/12/16 08:49:53 bodea Exp $";
 
 #include <time.h>
 #include <stdio.h>
@@ -21,9 +21,9 @@ char const *cvs_id_radius = "$Id: radius.c,v 1.12 2004/11/16 07:54:32 bodea Exp
 extern radiust *radius;
 extern sessiont *session;
 extern tunnelt *tunnel;
-extern u32 sessionid;
-extern struct configt *config;
+extern configt *config;
 extern int *radfds;
+extern ip_filtert *ip_filters;
 
 static const char *radius_state(int state)
 {
@@ -41,7 +41,7 @@ static const char *radius_state(int state)
 void initrad(void)
 {
 	int i;
-	LOG(3, 0, 0, 0, "Creating %d sockets for RADIUS queries\n", config->num_radfds);
+	LOG(3, 0, 0, "Creating %d sockets for RADIUS queries\n", config->num_radfds);
 	radfds = calloc(sizeof(int), config->num_radfds);
 	for (i = 0; i < config->num_radfds; i++)
 	{
@@ -52,17 +52,17 @@ void initrad(void)
 	}
 }
 
-void radiusclear(u16 r, sessionidt s)
+void radiusclear(uint16_t r, sessionidt s)
 {
 	if (s) session[s].radius = 0;
 	memset(&radius[r], 0, sizeof(radius[r])); // radius[r].state = RADIUSNULL;
 }
 
 
-static u16 get_free_radius()
+static uint16_t get_free_radius()
 {
 	int count;
-	static u32 next_radius_id = 0;
+	static uint32_t next_radius_id = 0;
 
 	for (count = MAXRADIUS; count > 0 ; --count)
 	{
@@ -76,24 +76,24 @@ static u16 get_free_radius()
 		}
 	}
 
-	LOG(0, 0, 0, 0, "Can't find a free radius session! This is very bad!\n");
+	LOG(0, 0, 0, "Can't find a free radius session! This is very bad!\n");
 	return 0;
 }
 
-u16 radiusnew(sessionidt s)
+uint16_t radiusnew(sessionidt s)
 {
-	u16 r = session[s].radius;
+	uint16_t r = session[s].radius;
 
 	/* re-use */
 	if (r)
 	{
-		LOG(3, 0, s, session[s].tunnel, "Re-used radius %d\n", r);
+		LOG(3, s, session[s].tunnel, "Re-used radius %d\n", r);
 		return r;
 	}
 
 	if (!(r = get_free_radius()))
 	{
-		LOG(1, 0, s, session[s].tunnel, "No free RADIUS sessions\n");
+		LOG(1, s, session[s].tunnel, "No free RADIUS sessions\n");
 		STAT(radius_overflow);
 		return 0;
 	};
@@ -104,18 +104,18 @@ u16 radiusnew(sessionidt s)
 	radius[r].state = RADIUSWAIT;
 	radius[r].retry = TIME + 1200; // Wait at least 120 seconds to re-claim this.
 
-	LOG(3,0,s, session[s].tunnel, "Allocated radius %d\n", r);
+	LOG(3, s, session[s].tunnel, "Allocated radius %d\n", r);
 	return r;
 }
 
 // Send a RADIUS request
-void radiussend(u16 r, u8 state)
+void radiussend(uint16_t r, uint8_t state)
 {
 	struct sockaddr_in addr;
-	u8 b[4096];            // RADIUS packet
+	uint8_t b[4096];            // RADIUS packet
 	char pass[129];
 	int pl;
-	u8 *p;
+	uint8_t *p;
 	sessionidt s;
 
 	CSTAT(call_radiussend);
@@ -123,12 +123,12 @@ void radiussend(u16 r, u8 state)
 	s = radius[r].session;
 	if (!config->numradiusservers)
 	{
-		LOG(0, 0, s, session[s].tunnel, "No RADIUS servers\n");
+		LOG(0, s, session[s].tunnel, "No RADIUS servers\n");
 		return;
 	}
 	if (!*config->radiussecret)
 	{
-		LOG(0, 0, s, session[s].tunnel, "No RADIUS secret\n");
+		LOG(0, s, session[s].tunnel, "No RADIUS secret\n");
 		return;
 	}
 
@@ -143,9 +143,10 @@ void radiussend(u16 r, u8 state)
 		radius[r].try = 0;
 	radius[r].state = state;
 	radius[r].retry = backoff(radius[r].try++);
-	LOG(4, 0, s, session[s].tunnel, "Send RADIUS id %d sock %d state %s try %d\n",
-			r >> RADIUS_SHIFT, r & RADIUS_MASK,
-			radius_state(radius[r].state), radius[r].try);
+	LOG(4, s, session[s].tunnel, "Send RADIUS id %d sock %d state %s try %d\n",
+		r >> RADIUS_SHIFT, r & RADIUS_MASK,
+		radius_state(radius[r].state), radius[r].try);
+
 	if (radius[r].try > config->numradiusservers * 2)
 	{
 		if (s)
@@ -154,7 +155,7 @@ void radiussend(u16 r, u8 state)
 				sessionshutdown(s, "RADIUS timeout");
 			else
 			{
-				LOG(1, 0, s, session[s].tunnel, "RADIUS timeout, but in state %s so don't timeout session\n",
+				LOG(1, s, session[s].tunnel, "RADIUS timeout, but in state %s so don't timeout session\n",
 					radius_states[state]);
 				radiusclear(r, s);
 			}
@@ -179,7 +180,7 @@ void radiussend(u16 r, u8 state)
 			b[0] = 4;               // accounting request
 			break;
 		default:
-			LOG(0, 0, 0, 0, "Unknown radius state %d\n", state);
+			LOG(0, 0, 0, "Unknown radius state %d\n", state);
 	}
 	b[1] = r >> RADIUS_SHIFT;       // identifier
 	memcpy(b + 4, radius[r].auth, 16);
@@ -244,7 +245,7 @@ void radiussend(u16 r, u8 state)
 	{                          // accounting
 		*p = 40;                // accounting type
 		p[1] = 6;
-		*(u32 *) (p + 2) = htonl((state == RADIUSSTART) ? 1 : 2);
+		*(uint32_t *) (p + 2) = htonl((state == RADIUSSTART) ? 1 : 2);
 		p += p[1];
 		if (s)
 		{
@@ -256,30 +257,30 @@ void radiussend(u16 r, u8 state)
 			{                // stop
 				*p = 42;      // input octets
 				p[1] = 6;
-				*(u32 *) (p + 2) = htonl(session[s].cin);
+				*(uint32_t *) (p + 2) = htonl(session[s].cin);
 				p += p[1];
 				*p = 43;      // output octets
 				p[1] = 6;
-				*(u32 *) (p + 2) = htonl(session[s].cout);
+				*(uint32_t *) (p + 2) = htonl(session[s].cout);
 				p += p[1];
 				*p = 46;      // session time
 				p[1] = 6;
-				*(u32 *) (p + 2) = htonl(time(NULL) - session[s].opened);
+				*(uint32_t *) (p + 2) = htonl(time(NULL) - session[s].opened);
 				p += p[1];
 				*p = 47;      // input packets
 				p[1] = 6;
-				*(u32 *) (p + 2) = htonl(session[s].pin);
+				*(uint32_t *) (p + 2) = htonl(session[s].pin);
 				p += p[1];
 				*p = 48;      // output spackets
 				p[1] = 6;
-				*(u32 *) (p + 2) = htonl(session[s].pout);
+				*(uint32_t *) (p + 2) = htonl(session[s].pout);
 				p += p[1];
 			}
 			else
 			{                // start
 				*p = 41;      // delay
 				p[1] = 6;
-				*(u32 *) (p + 2) = htonl(time(NULL) - session[s].opened);
+				*(uint32_t *) (p + 2) = htonl(time(NULL) - session[s].opened);
 				p += p[1];
 			}
 		}
@@ -288,14 +289,14 @@ void radiussend(u16 r, u8 state)
 	{
 		*p = 5; // NAS-Port
 		p[1] = 6;
-		*(u32 *) (p + 2) = htonl(s);
+		*(uint32_t *) (p + 2) = htonl(s);
 		p += p[1];
 	}
 	if (s && session[s].ip)
 	{
 		*p = 8;			// Framed-IP-Address
 		p[1] = 6;
-		*(u32 *) (p + 2) = htonl(session[s].ip);
+		*(uint32_t *) (p + 2) = htonl(session[s].ip);
 		p += p[1];
 	}
 	if (*session[s].called)
@@ -322,11 +323,11 @@ void radiussend(u16 r, u8 state)
 	// NAS-IP-Address
 	*p = 4;
 	p[1] = 6;
-	*(u32 *)(p + 2) = config->bind_address;
+	*(uint32_t *)(p + 2) = config->bind_address;
 	p += p[1];
 
 	// All AVpairs added
-	*(u16 *) (b + 2) = htons(p - b);
+	*(uint16_t *) (b + 2) = htons(p - b);
 	if (state != RADIUSAUTH)
 	{
 	    // Build auth for accounting packet
@@ -344,14 +345,14 @@ void radiussend(u16 r, u8 state)
 	}
 	memset(&addr, 0, sizeof(addr));
 	addr.sin_family = AF_INET;
-	*(u32 *) & addr.sin_addr = config->radiusserver[(radius[r].try - 1) % config->numradiusservers];
+	*(uint32_t *) & addr.sin_addr = config->radiusserver[(radius[r].try - 1) % config->numradiusservers];
 	{
-	    // get radius port
-	    u16	port = config->radiusport[(radius[r].try - 1) % config->numradiusservers];
-	    // no need to define the accounting port for itself:
-	    //  the accounting port is as far as I know always one more
-	    //  than the auth port    JK 20040713
-	    addr.sin_port = htons((state == RADIUSAUTH) ? port : port+1);
+		// get radius port
+		uint16_t port = config->radiusport[(radius[r].try - 1) % config->numradiusservers];
+		// no need to define the accounting port for itself:
+		//  the accounting port is as far as I know always one more
+		//  than the auth port    JK 20040713
+		addr.sin_port = htons((state == RADIUSAUTH) ? port : port+1);
 	}
 
 	LOG_HEX(5, "RADIUS Send", b, (p - b));
@@ -359,15 +360,15 @@ void radiussend(u16 r, u8 state)
 }
 
 // process RADIUS response
-void processrad(u8 *buf, int len, char socket_index)
+void processrad(uint8_t *buf, int len, char socket_index)
 {
-	u8 b[MAXCONTROL];
+	uint8_t b[MAXCONTROL];
 	MD5_CTX ctx;
-	u16 r;
+	uint16_t r;
 	sessionidt s;
 	tunnelidt t = 0;
 	hasht hash;
-	u8 routes = 0;
+	uint8_t routes = 0;
 
 	int r_code, r_id ; // Radius code.
 
@@ -378,24 +379,24 @@ void processrad(u8 *buf, int len, char socket_index)
 	CSTAT(call_processrad);
 
 	LOG_HEX(5, "RADIUS Response", buf, len);
-	if (len < 20 || len < ntohs(*(u16 *) (buf + 2)))
+	if (len < 20 || len < ntohs(*(uint16_t *) (buf + 2)))
 	{
-		LOG(1, 0, 0, 0, "Duff RADIUS response length %d\n", len);
+		LOG(1, 0, 0, "Duff RADIUS response length %d\n", len);
 		return ;
 	}
-	len = ntohs(*(u16 *) (buf + 2));
+	len = ntohs(*(uint16_t *) (buf + 2));
 	r = socket_index | (r_id << RADIUS_SHIFT);
 	s = radius[r].session;
-	LOG(3, 0, s, session[s].tunnel, "Received %s, radius %d response for session %u (code %d, id %d)\n",
+	LOG(3, s, session[s].tunnel, "Received %s, radius %d response for session %u (code %d, id %d)\n",
 			radius_states[radius[r].state], r, s, r_code, r_id);
 	if (!s && radius[r].state != RADIUSSTOP)
 	{
-		LOG(1, 0, s, session[s].tunnel, "   Unexpected RADIUS response\n");
+		LOG(1, s, session[s].tunnel, "   Unexpected RADIUS response\n");
 		return;
 	}
 	if (radius[r].state != RADIUSAUTH && radius[r].state != RADIUSSTART && radius[r].state != RADIUSSTOP)
 	{
-		LOG(1, 0, s, session[s].tunnel, "   Unexpected RADIUS response\n");
+		LOG(1, s, session[s].tunnel, "   Unexpected RADIUS response\n");
 		return;
 	}
 	t = session[s].tunnel;
@@ -408,46 +409,44 @@ void processrad(u8 *buf, int len, char socket_index)
 	do {
 		if (memcmp(hash, buf + 4, 16))
 		{
-			LOG(0, 0, s, session[s].tunnel, "   Incorrect auth on RADIUS response!! (wrong secret in radius config?)\n");
+			LOG(0, s, session[s].tunnel, "   Incorrect auth on RADIUS response!! (wrong secret in radius config?)\n");
 			return; // Do nothing. On timeout, it will try the next radius server.
 		}
 		if ((radius[r].state == RADIUSAUTH && *buf != 2 && *buf != 3) ||
 			((radius[r].state == RADIUSSTART || radius[r].state == RADIUSSTOP) && *buf != 5))
 		{
-			LOG(1, 0, s, session[s].tunnel, "   Unexpected RADIUS response %d\n", *buf);
+			LOG(1, s, session[s].tunnel, "   Unexpected RADIUS response %d\n", *buf);
 			return; // We got something we didn't expect. Let the timeouts take
 				// care off finishing the radius session if that's really correct.
 		}
 		if (radius[r].state == RADIUSAUTH)
 		{
-			LOG(4, 0, s, session[s].tunnel, "   Original response is \"%s\"\n", (*buf == 2) ? "accept" : "reject");
+			LOG(4, s, session[s].tunnel, "   Original response is \"%s\"\n", (*buf == 2) ? "accept" : "reject");
 			// process auth response
 			if (radius[r].chap)
 			{
 				// CHAP
-				u8 *p = makeppp(b, sizeof(b), 0, 0, t, s, PPPCHAP);
-				if (!p) {
-					return;	// Abort!
-				}
+				uint8_t *p = makeppp(b, sizeof(b), 0, 0, t, s, PPPCHAP);
+				if (!p) return;	// Abort!
+
 				{
 					struct param_post_auth packet = { &tunnel[t], &session[s], session[s].user, (*buf == 2), PPPCHAP };
 					run_plugins(PLUGIN_POST_AUTH, &packet);
 					*buf = packet.auth_allowed ? 2 : 3;
 				}
 
-				LOG(3, 0, s, session[s].tunnel, "   CHAP User %s authentication %s.\n", session[s].user,
+				LOG(3, s, session[s].tunnel, "   CHAP User %s authentication %s.\n", session[s].user,
 						(*buf == 2) ? "allowed" : "denied");
 				*p = (*buf == 2) ? 3 : 4;     // ack/nak
 				p[1] = radius[r].id;
-				*(u16 *) (p + 2) = ntohs(4); // no message
+				*(uint16_t *) (p + 2) = ntohs(4); // no message
 				tunnelsend(b, (p - b) + 4, t); // send it
 			}
 			else
 			{
 				// PAP
-				u8 *p = makeppp(b, sizeof(b), 0, 0, t, s, PPPPAP);
-				if (!p)
-					return;		// Abort!
+				uint8_t *p = makeppp(b, sizeof(b), 0, 0, t, s, PPPPAP);
+				if (!p) return;		// Abort!
 
 				{
 					struct param_post_auth packet = { &tunnel[t], &session[s], session[s].user, (*buf == 2), PPPPAP };
@@ -455,12 +454,12 @@ void processrad(u8 *buf, int len, char socket_index)
 					*buf = packet.auth_allowed ? 2 : 3;
 				}
 
-				LOG(3, 0, s, session[s].tunnel, "   PAP User %s authentication %s.\n", session[s].user,
+				LOG(3, s, session[s].tunnel, "   PAP User %s authentication %s.\n", session[s].user,
 						(*buf == 2) ? "allowed" : "denied");
 				// ack/nak
 				*p = *buf;
 				p[1] = radius[r].id;
-				*(u16 *) (p + 2) = ntohs(5);
+				*(uint16_t *) (p + 2) = ntohs(5);
 				p[4] = 0; // no message
 				tunnelsend(b, (p - b) + 5, t); // send it
 			}
@@ -469,37 +468,43 @@ void processrad(u8 *buf, int len, char socket_index)
 			{
 				// Login successful
 				// Extract IP, routes, etc
-				u8 *p = buf + 20;
-				u8 *e = buf + len;
-				for (p = buf + 20; p < e && p[1]; p += p[1])
+				uint8_t *p = buf + 20;
+				uint8_t *e = buf + len;
+				for (; p + 2 <= e && p[1] && p + p[1] <= e; p += p[1])
 				{
 					if (*p == 8)
 					{
-						// Statically assigned address
-						LOG(3, 0, s, session[s].tunnel, "   Radius reply contains IP address %s\n", inet_toa(*(u32 *) (p + 2)));
-						session[s].ip = ntohl(*(u32 *) (p + 2));
+						// Framed-IP-Address
+					    	if (p[1] < 6) continue;
+						session[s].ip = ntohl(*(uint32_t *) (p + 2));
 						session[s].ip_pool_index = -1;
+						LOG(3, s, session[s].tunnel, "   Radius reply contains IP address %s\n",
+							fmtaddr(htonl(session[s].ip), 0));
 					}
 					else if (*p == 135)
 					{
 						// DNS address
-						LOG(3, 0, s, session[s].tunnel, "   Radius reply contains primary DNS address %s\n", inet_toa(*(u32 *) (p + 2)));
-						session[s].dns1 = ntohl(*(u32 *) (p + 2));
+					    	if (p[1] < 6) continue;
+						session[s].dns1 = ntohl(*(uint32_t *) (p + 2));
+						LOG(3, s, session[s].tunnel, "   Radius reply contains primary DNS address %s\n",
+							fmtaddr(htonl(session[s].dns1), 0));
 					}
 					else if (*p == 136)
 					{
 						// DNS address
-						LOG(3, 0, s, session[s].tunnel, "   Radius reply contains secondary DNS address %s\n", inet_toa(*(u32 *) (p + 2)));
-						session[s].dns2 = ntohl(*(u32 *) (p + 2));
+					    	if (p[1] < 6) continue;
+						session[s].dns2 = ntohl(*(uint32_t *) (p + 2));
+						LOG(3, s, session[s].tunnel, "   Radius reply contains secondary DNS address %s\n",
+							fmtaddr(htonl(session[s].dns2), 0));
 					}
 					else if (*p == 22)
 					{
-						// framed-route
-						ipt ip = 0, mask = 0;
-						u8 u = 0;
-						u8 bits = 0;
-						u8 *n = p + 2;
-						u8 *e = p + p[1];
+						// Framed-Route
+						in_addr_t ip = 0, mask = 0;
+						uint8_t u = 0;
+						uint8_t bits = 0;
+						uint8_t *n = p + 2;
+						uint8_t *e = p + p[1];
 						while (n < e && (isdigit(*n) || *n == '.'))
 						{
 							if (*n == '.')
@@ -525,71 +530,110 @@ void processrad(u8 *buf, int len, char socket_index)
 							mask = 0xFFFF0000;
 						else
 							mask = 0xFFFFFF00;
+
 						if (routes == MAXROUTE)
 						{
-							LOG(1, 0, s, session[s].tunnel, "   Too many routes\n");
+							LOG(1, s, session[s].tunnel, "   Too many routes\n");
 						}
 						else if (ip)
 						{
-							char *ips, *masks;
-							ips = strdup(inet_toa(htonl(ip)));
-							masks = strdup(inet_toa(htonl(mask)));
-							LOG(3, 0, s, session[s].tunnel, "   Radius reply contains route for %s/%s\n", ips, masks);
-							free(ips);
-							free(masks);
+							LOG(3, s, session[s].tunnel, "   Radius reply contains route for %s/%s\n",
+								fmtaddr(htonl(ip), 0), fmtaddr(htonl(mask), 1));
+							
 							session[s].route[routes].ip = ip;
 							session[s].route[routes].mask = mask;
 							routes++;
 						}
 					}
-					else if (*p == 26)
+					else if (*p == 11)
+					{
+					    	// Filter-Id
+					    	char *filter = p + 2;
+						int l = p[1] - 2;
+						char *suffix;
+						uint8_t *f = 0;
+						int i;
+
+						LOG(3, s, session[s].tunnel, "   Radius reply contains Filter-Id \"%.*s\"\n", l, filter);
+						if ((suffix = memchr(filter, '.', l)))
+						{
+							int b = suffix - filter;
+							if (l - b == 3 && !memcmp("in", suffix+1, 2))
+								f = &session[s].filter_in;
+							else if (l - b == 4 && !memcmp("out", suffix+1, 3))
+								f = &session[s].filter_out;
+
+							l = b;
+						}
+
+						if (!f)
+						{
+							LOG(3, s, session[s].tunnel, "    Invalid filter\n");
+							continue;
+						}
+
+						for (*f = 0, i = 0; !*f && i < MAXFILTER; i++)
+							if (strlen(ip_filters[i].name) == l &&
+							    !strncmp(ip_filters[i].name, filter, l))
+								*f = i + 1;
+
+						if (*f)
+							ip_filters[*f - 1].used++;
+						else
+							LOG(3, s, session[s].tunnel, "    Unknown filter\n");
+
+					}
+					else if (*p == 26 && p[1] >= 7)
 					{
 						// Vendor-Specific Attribute
 						int vendor = ntohl(*(int *)(p + 2));
 						char attrib = *(p + 6);
 						char attrib_length = *(p + 7) - 2;
-						LOG(3, 0, s, session[s].tunnel, "   Radius reply contains Vendor-Specific. Vendor=%d Attrib=%d Length=%d\n", vendor, attrib, attrib_length);
-						if (attrib_length == 0) continue;
-						if (attrib != 1)
-							LOG(3, 0, s, session[s].tunnel, "      Unknown vendor-specific\n");
-						else
+						char *avpair, *value, *key, *newp;
+
+						LOG(3, s, session[s].tunnel, "   Radius reply contains Vendor-Specific.  Vendor=%d Attrib=%d Length=%d\n", vendor, attrib, attrib_length);
+						if (vendor != 9 || attrib != 1)
 						{
-							char *avpair, *value, *key, *newp;
-							avpair = key = calloc(attrib_length + 1, 1);
-							memcpy(avpair, p + 8, attrib_length);
-							LOG(3, 0, s, session[s].tunnel, "      Cisco-Avpair value: %s\n", avpair);
-							do {
-								value = strchr(key, '=');
-								if (!value) break;
-								*value++ = 0;
-
-								// Trim quotes off reply string
-								if (*value == '\'' || *value == '\"')
-								{
-									char *x;
-									value++;
-									x = value + strlen(value) - 1;
-									if (*x == '\'' || *x == '\"')
-										*x = 0;
-								}
-
-								// Run hooks
-								newp = strchr(value, ',');
-								if (newp) *newp++ = 0;
-								{
-									struct param_radius_response p = { &tunnel[session[s].tunnel], &session[s], key, value };
-									run_plugins(PLUGIN_RADIUS_RESPONSE, &p);
-								}
-								key = newp;
-							} while (newp);
-							free(avpair);
+							LOG(3, s, session[s].tunnel, "      Unknown vendor-specific\n");
+							continue;
 						}
+
+						if (attrib_length < 0) continue;
+
+						avpair = key = calloc(attrib_length + 1, 1);
+						memcpy(avpair, p + 8, attrib_length);
+						LOG(3, s, session[s].tunnel, "      Cisco-Avpair value: %s\n", avpair);
+						do {
+							value = strchr(key, '=');
+							if (!value) break;
+							*value++ = 0;
+
+							// Trim quotes off reply string
+							if (*value == '\'' || *value == '\"')
+							{
+								char *x;
+								value++;
+								x = value + strlen(value) - 1;
+								if (*x == '\'' || *x == '\"')
+									*x = 0;
+							}
+
+							// Run hooks
+							newp = strchr(value, ',');
+							if (newp) *newp++ = 0;
+							{
+								struct param_radius_response p = { &tunnel[session[s].tunnel], &session[s], key, value };
+								run_plugins(PLUGIN_RADIUS_RESPONSE, &p);
+							}
+							key = newp;
+						} while (newp);
+						free(avpair);
 					}
 				}
 			}
 			else if (*buf == 3)
 			{
-				LOG(2, 0, s, session[s].tunnel, "   Authentication denied for %s\n", session[s].user);
+				LOG(2, s, session[s].tunnel, "   Authentication denied for %s\n", session[s].user);
 //FIXME: We should tear down the session here!
 				break;
 			}
@@ -597,12 +641,12 @@ void processrad(u8 *buf, int len, char socket_index)
 			if (!session[s].dns1 && config->default_dns1)
 			{
 				session[s].dns1 = htonl(config->default_dns1);
-				LOG(3, 0, s, t, "   Sending dns1 = %s\n", inet_toa(config->default_dns1));
+				LOG(3, s, t, "   Sending dns1 = %s\n", fmtaddr(config->default_dns1, 0));
 			}
 			if (!session[s].dns2 && config->default_dns2)
 			{
 				session[s].dns2 = htonl(config->default_dns2);
-				LOG(3, 0, s, t, "   Sending dns2 = %s\n", inet_toa(config->default_dns2));
+				LOG(3, s, t, "   Sending dns2 = %s\n", fmtaddr(config->default_dns2, 0));
 			}
 
 			// Valid Session, set it up
@@ -612,7 +656,7 @@ void processrad(u8 *buf, int len, char socket_index)
 		else
 		{
 				// An ack for a stop or start record.
-			LOG(3, 0, s, t, "   RADIUS accounting ack recv in state %s\n", radius_states[radius[r].state]);
+			LOG(3, s, t, "   RADIUS accounting ack recv in state %s\n", radius_states[radius[r].state]);
 			break;
 		}
 	} while (0);
@@ -622,7 +666,7 @@ void processrad(u8 *buf, int len, char socket_index)
 }
 
 // Send a retry for RADIUS/CHAP message
-void radiusretry(u16 r)
+void radiusretry(uint16_t r)
 {
 	sessionidt s = radius[r].session;
 	tunnelidt t = 0;
@@ -654,7 +698,7 @@ void radiusretry(u16 r)
 		case RADIUSWAIT:           // waiting timeout before available, in case delayed reply from RADIUS server
 			// free up RADIUS task
 			radiusclear(r, s);
-			LOG(3, 0, s, session[s].tunnel, "Freeing up radius session %d\n", r);
+			LOG(3, s, session[s].tunnel, "Freeing up radius session %d\n", r);
 			break;
 	}
 }
