@@ -1,7 +1,7 @@
 #ifndef __UTIL_H__
 #define __UTIL_H__
 
-char *inet_toa(unsigned long addr);
+char *fmtaddr(in_addr_t addr, int n);
 void *shared_malloc(unsigned int size);
 pid_t fork_and_close(void);
 
